module M2
   implicit  none
   public :: CONVERT

contains

!! This function converts an integer to a string.
!! It uses programmed conversion, which is usually quicker than
!! conversion using internal I/O.
!! The string is right-adjusted in a field of 20 characters.
!! The 20 characters may be adjusted to suit your machine.
!! Allow at least 12 characters for a 32-bit machine.
!! If the string is not long enough, the entire field is replaced
!! with asterisks (just like internal I/O would).
function CONVERT (K) result (String)
   !! INCOMING: K = an integer.
   !! RETURNS:  A character string version of the binary integer dummy argument K.

   integer, intent (in)            :: K

   character (len=20)              :: String

   integer                         :: Quotient, Rem, J, N

   String = " "
   N = K
   do J = len(String), 2, -1
      Quotient = N / 10                           !! Divide by 10, and collect the quotient . . .
      Rem = N - Quotient*10                       !! . . . and the remainder.
      String(J:J) = char( ichar("0") + abs(Rem) ) !! Convert the remainder to a character digit.
      if (Quotient == 0) then                     !! Conversion of the integer K is complete.
         if (K < 0) then                          !! The original integer K was negative, so . . .
            String(J-1:J-1) = "-"                 !! . . . prefix a minus sign.
         end  if
         return
      end  if
      N = Quotient
   end  do

   String = repeat ("*", len(String) )            !! The integer won't fit in the given string.

end  function  CONVERT

end  module  M2
