!! This module procedure implements automatic conversion from integer to character on assignment.
!! It also implements automatic conversion from character string to integer on assignment,
!! and conversion from character to real, on assignment.
!! The module overloads the new conversions on the assignment operator (=).
module  CONVERSION
   implicit none
   public :: INTEGER_TO_CHAR, CHAR_TO_INTEGER, CHAR_TO_FLOAT
   public :: assignment (=)

   interface  assignment (=)
      module  procedure  INTEGER_TO_CHAR
      module  procedure  CHAR_TO_INTEGER
      module  procedure  CHAR_TO_FLOAT
   end  interface

contains

   !! This subroutine converts an integer K to character form, storing the character
   !! equivalent left adjusted in the variable C.
   !! If C is not long enough, the conversion is terminated with an error message.
   subroutine  INTEGER_TO_CHAR (C, K)
      !! INCOMING: C = a character variable to receive the converted integer;
      !!           K = the integer to be converted to a character string.
      character (len=*), intent (out) :: C
      integer, intent (in)            :: K

      character (len=25)              :: B
      integer                         :: J

      write (unit=B, fmt="(i25)" ) K              !! Convert to CHARACTER.

      do J = 1, len(B)                            !! Loop to find the first non-blank character.
         if (B(J:J) /= " ") then
            exit
         end  if
      end  do

      if (len_trim(B(J: )) > len(C)) then
         print *, "Character variable is not long enough to receive the converted integer ", K
         print *, "The variable on the left of the assignment has a length of ", len(C)
         stop
      end  if
      C = B(J: )                                  !! Left adjusts the digits in C.
   end  subroutine  INTEGER_TO_CHAR

   !! This subroutine converts a character string to an integer.
   subroutine  CHAR_TO_INTEGER (K, C)
      !! INCOMING: K = an integer variable to receive the converted character string;
      !!           C = the character variable to be converted to an integer.
      integer, intent (out)          :: K
      character (len=*), intent (in) :: C

      integer                        :: L

      L = len (C)
      if (L < 1) then
         print *, "Character value has null length.  Cannot convert to integer."
         stop
      end  if
      read (unit=C, fmt=*) K
   end  subroutine  CHAR_TO_INTEGER

   !! This subroutine converts a character string to a floating-point value.
   subroutine  CHAR_TO_FLOAT (A, C)
      !! INCOMING:   A  = a floating=point variable to receive the converted character string;
      !!       C  = the character variable to be converted to a floating-point.
      real, intent (out)             :: A
      character (len=*), intent (in) :: C

      integer                        :: L

      L = len (C)
      if (L < 1) then
         print *, "Character value has null length.  Cannot convert to floating-point."
         stop
      end  if

      read (unit=C, fmt=*) A
   end  subroutine  CHAR_TO_FLOAT

end  module  CONVERSION
