!! This module procedure implements the three-argument version of the built-in function index.
module  INDEX_MODULE
   implicit none
   public                          :: index, INDEX3
   intrinsic  index

   interface  index
      module  procedure  INDEX3
   end  interface

contains

   !! This function searches the first string String, to ascertain whether the second string
   !! Sub exists within it.  If it does, the function returns the position of the left-most
   !! character that matches.  The search commences from character position Position in String.
   function  INDEX3 (String, Sub, Position) result (Pos)
      !! INCOMING: String   = the string to be searched;
      !!           Sub      = contains the string to look for;
      !!           Position = where to start the search (measured from the left-hand end of
      !!                      String).
      character (len=*), intent (in)   :: String, Sub
      integer, intent (in)         :: Position
      integer                      :: Pos

      integer                      :: K

      if (len (String) == 0) then                 !! There's nothing to search.
         Pos = 0
         return
      end  if
      if (len (Sub) == 0) then                    !! Nothing with which to search.
         Pos = 0
         return
      end  if

      K = index ( String (Position: ), Sub)       !! Start the search from position Position.
      if (K == 0) then
         Pos = 0
         return
      end  if
      Pos = Position + K - 1
      return
   end  function  INDEX3

end  module  INDEX_MODULE

