module  INDEX_REVERSE_MODULE
   implicit none
   public                          :: INDEXR, INDEXR_all, INDEXR_sub

   interface  INDEXR
      module  procedure  INDEXR_all
      module  procedure  INDEXR_sub
   end  interface

contains

   !! This function searches the first string String, to ascertain whether the second string
   !! Sub exists within it.  If it does, the function returns the position of the right-most
   !! character that matches.  The search commences from right to left.
   function  INDEXR_all (String, Sub) result (Pos)
      !! INCOMING: String = the string to be searched;
      !!           Sub    = contains the string to look for;
      character (len=*), intent (in) :: String, Sub
      integer                        :: Pos

      character (len=1)              :: CK
      integer                        :: K
      integer                        :: LP, LS

      LP = len (Sub)
      LS = len (String)
      if (LP + LS == 0) then                      !! There's nothing to search.
         Pos = 0
         return
      end  if

      CK = Sub(1:1)
      if (LP == 1) then                           !! A search for 1 character is faster.
         do K = LS, 1, -1                         !! A loop to scan String backwards . . .
            if (String(K:K) == CK) then
               Pos = K
               return
            end  if
         end  do
         Pos = 0
         return
      end  if

      K = LS-LP+2                                 !! A loop to scan String backwards . . .
      do
         do K = K-1, 1, -1                        !! A fast inner loop to look for a starting character.
            if (CK == String(K:K) ) then
               exit
            end  if
         end  do
         if (K > 0) then
            if (String(K:K+LP-1) == Sub) then
               Pos = K
               return                             !! We had a match at position K.
            end  if
         end  if
         if (K < 0)  then
            exit
         end  if
      end  do

      Pos = 0                                     !! No luck at all.  Go back empty-handed.
   end  function  INDEXR_all

   !! This function searches the first string String, to ascertain whether the second string
   !! Sub exists within it.  If it does, the function returns the position of the right-most
   !! character that matches.  The search commences from position Position, from right
   !! to left.
   !! Note that the value of Position must be such that the string Sub will fit within
   !! String.  If it does not, the search will commence from position
   !! len(String) - len(Sub) + 1, that is, with Sub right-adjusted alongside String.
   function  INDEXR_sub (String, Sub, Position) result (Pos)
      !! INCOMING: String   = the string to be searched;
      !!           Sub      = contains the string to look for;
      !!           Position = where to start the search (measured from the left-hand end of
      !!                      String).
      character (len=*), intent (in) :: String, Sub
      integer, intent (in)           :: Position
      integer                        :: Pos

      character (len=1)              :: CK
      integer                        :: K
      integer                        :: LP, LS

      LP = len (Sub)
      LS = len (String)
      if ( (Position > LS) .or. (Position < 0) ) then
         Pos = -1                                 !! Indicate failure.
         return
      end  if
      if (LP + LS == 0) then                      !! There's nothing to search.
         Pos = 0
         return
      end  if

      CK = Sub(1:1)
      if (LP == 1) then                           !! A search for 1 character is faster.
         do K = Position, 1, -1                   !! A loop to scan String backwards . . .
            if (String(K:K) == CK) then
               Pos = K
               return
            end  if
         end  do
         Pos = 0
         return
      end  if

      K = min(LS-LP+2, Position+1)

      do
         do K = K-1, 1, -1
 
           if (CK == String(K:K) ) then
               exit
            end  if
         end  do                                  !! A fast inner loop to look for a starting character.
         if (K > 0) then
            if (String(K:K+LP-1) == Sub) then
               Pos = K
               return                             !! We had a match at position K.
            end  if
         end  if

         if (K <= 1)  then
            exit
         end if
      end  do

      Pos = 0                                     !! No luck at all.  Go back empty-handed.
   end  function  INDEXR_sub

end  module  INDEX_REVERSE_MODULE
