module  RESHAPE_MODULE
   !! This module contains reshape functions (which reshape a vector to a matrix,
   !! in row-major order).
   implicit  none
   public                     :: RESHAPE_BY_ROWS, RESHAPE_integer, RESHAPE_real, RESHAPE_double
   public                     :: RESHAPE_logical
   integer, parameter, public :: Double = selected_real_kind (14)

   interface  RESHAPE_BY_ROWS
      module  procedure  RESHAPE_integer
      module  procedure  RESHAPE_real
      module  procedure  RESHAPE_double
      module  procedure  RESHAPE_logical
   end  interface

contains

   !! This function converts a vector to a matrix.  The elements in the vector
   !! are taken to fill the matrix in row-major order.
   function RESHAPE_integer (A, Shape_spec) result (C)
      integer, dimension ( : ), intent (in)  :: A, Shape_spec
      integer, dimension ( Shape_spec(1), Shape_spec(2) ) :: C

      integer                      :: M, N, J, K

      M = Shape_spec(1)                           !! The number of rows in the result matrix.
      N = Shape_spec(2)                           !! The number of columns in the result matrix.

      K = 1
      do J = 1, M                                 !! A loop to build C a row at a time . . .
         C(J, : ) = A(K: K+N-1)                   !! Extract N elements from A, making one row of C.
         K = K + N
      end  do

   end  function RESHAPE_integer

   !! This function converts a vector to a matrix.  The elements in the vector
   !! are taken to fill the matrix in row-major order.
   function  RESHAPE_real (A, Shape_spec) result (C)
      real, dimension ( : ), intent (in)     :: A
      integer, dimension ( : ), intent (in)  :: Shape_spec
      real, dimension ( Shape_spec(1), Shape_spec(2) ) :: C

      integer                      :: M, N, J, K

      M = Shape_spec(1)                           !! The number of rows in the result matrix.
      N = Shape_spec(2)                           !! The number of columns in the result matrix.

      K = 1
      do J = 1, M                                 !! A loop to build C a row at a time . . .
         C(J, : ) = A(K: K+N-1)                   !! Extract N elements from A, making one row of C.
         K = K + N
      end  do

   end  function  RESHAPE_real

   !! This function converts a vector to a matrix.  The elements in the vector
   !! are taken to fill the matrix in row-major order.
   function  RESHAPE_double (A, Shape_spec) result (C)
      real (kind=Double), dimension ( : ), intent (in) :: A
      integer, dimension ( : ), intent (in)            :: Shape_spec
      real (kind=Double), dimension ( Shape_spec(1), Shape_spec(2) ) :: C

      integer                       :: M, N, J, K

      M = Shape_spec(1)                           !! The number of rows in the result matrix.
      N = Shape_spec(2)                           !! The number of columns in the result matrix.

      K = 1
      do J = 1, M                                 !! A loop to build C a row at a time . . .
         C(J, : ) = A(K: K+N-1)                   !! Extract N elements from A, making one row of C.
         K = K + N
      end  do

   end  function RESHAPE_double

   !! This function converts a vector to a matrix.  The elements in the vector
   !! are taken to fill the matrix in row-major order.
   function  RESHAPE_logical (A, Shape_spec) result (C)
      logical, dimension ( : ), intent (in)     :: A
      integer, dimension ( : ), intent (in)     :: Shape_spec
      logical, dimension ( Shape_spec(1), Shape_spec(2) ) :: C

      integer                      :: M, N, J, K

      M = Shape_spec(1)                           !! The number of rows in the result matrix.
      N = Shape_spec(2)                           !! The number of columns in the result matrix.

      K = 1
      do J = 1, M                                 !! A loop to build C a row at a time . . .
         C(J, : ) = A(K: K+N-1)                   !! Extract N elements from A, making one row of C.
         K = K + N
      end  do

   end  function RESHAPE_logical

end  module  RESHAPE_MODULE
