module M3
   implicit  none
   public :: REVERSE_BYTES

contains

!! This function reverses the bytes of a binary integer
!! (it is required that the number of bits in a word is a multiple of 8).
!! It is used to convert binary data written by a machine that writes integers in the reverse order
!! compared to the current machine.
function  REVERSE_BYTES (K) result (K_reversed)
   !! INCOMING: K = a 32-bit binary integer.
   !! RETURNS:      the integer K with the bytes reversed.
   integer, intent (in)            :: K

   integer                         :: K_reversed

   integer                         :: J, L
   integer, parameter              :: BS8 = bit_size(K)/8, &
                                      BS4 = bit_size(K)/4, &
                                      BS2 = bit_size(K)/2, &
                                      BS = bit_size(K)

   select case (bit_size(K) )
      case (32)                                   !! Special case is fast for 32-bit integers.
         J = ishftc (K, BS4, BS2)                 !! Swap the two least-significant bytes.
         J = ishftc (J, BS2)                      !! Rotate 16 bit positions to the left.
         K_reversed = ishftc (J, BS4, BS2)        !! Swap the two least-significant bytes.
      case (64)                                   !! Special case is fast for 64-bit integers.
         J = ishftc (K, BS8, BS4)                 !! Swap the two least-significant bytes.
         J = ishftc (J, BS4, BS2)                 !! Swap the two least-significant 16-bit units.
         J = ishftc (J, BS8, BS4)                 !! Swap the two least-significant bytes.
         J = ishftc (J, BS2, BS)                  !! Rotate 32 places to the left, bringing the upper
                                                  !! 32 bits to the lower position.
         J = ishftc (J, BS8, BS4)                 !! Swap the two least-significant bytes.
         J = ishftc (J, BS4, BS2)                 !! Swap the two least-significant 16-bit units.
         K_reversed = ishftc (J, BS8, BS4)        !! Swap the two least-significant bytes.
      case (16)                                   !! Special case is fast for 16-bit integers.
         K_reversed = ishftc (K, BS2)             !! Swap the two least-significant bytes.
      case  default                               !! The general case.
         K_reversed = 0
         J = K
         do L = 1, bit_size(K)/8                  !! Once around for each byte.
            K_reversed = ishft(K_reversed, 8) &   !! Left shift, `  . . .
               + iand(J, 255)                     !! . . . & add in the extracted low 8 bits of the integer.
            J = ishft(J, -8)                      !! Right shift the integer.
         end  do
   end  select
end  function  REVERSE_BYTES

end  module  M3
