!! This module implements the SCAN function, which searches from any position in the parent
!! string.  It complements the existing SCAN built-in function.
module  SCAN_MODULE
   implicit none
   public                          :: scan, SCAN_sub
   intrinsic scan

   interface  scan
      module  procedure  SCAN_sub
   end  interface

contains

   !! This function searches the first string String for any characters given in the
   !! second string Sub.  If there are any such characters, the function returns the position
   !! of the left-most.
   !! The search is performed from left to right, commencing from character position Position.
   function  SCAN_sub (String, Sub, Position) result (Pos)
      !! INCOMING: String   = the string to be searched;
      !!           Sub      = contains characters to look for;
      !!           Position = where to start the search (measured from the left-hand end of
      !!                      String).
      character (len=*), intent (in) :: String, Sub
      integer, intent (in)           :: Position
      integer                        :: Pos

      integer                        :: K

      Pos = 0
      if (len (String) == 0) then
         return
      end  if
      K = scan ( String(Position: ), Sub)
      if (K == 0) then
         return
      end  if
      Pos = Position+K-1
   end  function  SCAN_sub

end  module  SCAN_MODULE

