module TRANSLATE_MODULE
   implicit  none
   public                          :: TRANSLATE

contains

   !! This function translates characters in String using a replacement list formed by
   !! From_List and To_List.  Suppose that a character in String exists in From_List.
   !! Then the character in To_List corresponding to that character in From_List
   !! replaces the one in String.
   function  TRANSLATE (String, From_List, To_List) result (Translated_String)
      character (len=*), intent (in)  :: String, From_List, To_List

      character (len=len(String) )    :: Translated_String

      character (len=len(From_List) ) :: To_List_Aux
      character (len=256)             :: New_Table
      integer                         :: J, K

      To_List_Aux = To_List                       !! To make sure that From_List and To_List are the
                                                  !! same length.  To_List_Aux is padded with blanks.

      !! PRE-PROCESSING SECTION
      !! Set up New_Table so that each character in String is replaced by itself.
      do J = 1, 256
         New_Table(J:J) = char(J-1)
      end  do

      !! For each character in the From_List, store (in New_Table) a character from the To_List.
      do J = 1, len(From_List)
         K = ichar (From_List(J:J)) + 1
         New_Table(K:K) = To_List_Aux(J:J)
      end  do

      !! TRANSLATING SECTION
      !! For each character in the String, index To_List to obtain the character and store it in the
      !! result.
      do J = 1, len (String)
         K = ichar(String(J:J) ) + 1
         Translated_String(J:J) = New_Table(K:K)
      end  do

      K = min( len(String)+1, len(Translated_String) )
      Translated_String(K: ) = " "
   end  function  TRANSLATE

end  module  TRANSLATE_MODULE

