module GROUP
   implicit  none
   public :: COCKTAIL

contains

!! The Cocktail-Shaker Sort.  Performs an Exchange Sort on the elements in an array A.
!! The scans alternate from the top to bottom, then bottom to top.  The scans continue until
!! the data are in order.
subroutine COCKTAIL (A)
   !! INCOMING:   A = an array whose elements are to be sorted;
   !! OUTGOING:   A = the sorted array.  Elements are in ascending order.

   integer, dimension ( : ), intent (in out) :: A

   integer                         :: N           !! The number of elements in array A.
   integer                         :: Temp
   integer                         :: J
   logical                         :: Sorted

   N = ubound(A, 1)                               !! The number of elements to be sorted.

   do
      Sorted = .true.                             !! Assume that they are sorted.
      do J = 1, N-1                               !! Top to bottom.
         if (A(J) > A(J+1)) then                  !! Swap elements to bring them to order.
            Temp = A(J)
            A(J) = A(J+1)
            A(J+1) = Temp
            Sorted = .false.                      !! Note that we swapped (raise the flag).
         end  if
      end  do

      if (Sorted) then
         return
      end  if

      do J = N-2, 1, -1                           !! Bottom to top.
         if (A(J) > A(J+1)) then                  !! Swap elements to bring them to order.
            Temp = A(J)
            A(J) = A(J+1)
            A(J+1) = Temp
            Sorted = .false.                      !! Note that we swapped (raise the flag).
         end  if
      end  do
      if (Sorted) then
         return
      end  if
   end  do

end  subroutine  COCKTAIL

end  module  GROUP
