module GROUP
   implicit  none
   public :: SHELL_SORT

contains
   !! This subroutine sorts an array by sorting continually changing subgroups.

   subroutine SHELL_SORT (Person)
      !! INCOMING: Person = an array whose elements are to be sorted.
      !! OUTGOING: Person = the sorted array.  Elements are in ascending order.
      use OUTLINE
      type (Personage), dimension ( : ), intent (in out) :: Person

      type (Personage)             :: Temp
      integer                      :: J, K, N, Gap

      N = ubound(Person, 1)                       !! The number of elements in the array Person.

      Gap = N/2                                   !! The initial gap is half the number of elements.
      do
         if (Gap <= 1) then                       !! We are done after the gap is 1, when all
            exit                                  !! elements are examined in the next loop:
         end  if
         !! Perform a Ranking Sort on selected elements.
         do  K = 1,  N-Gap
            if (Person(K)%Name > Person(K+Gap)%Name) then
                                                  !! Select elements every Gap positions apart . .

               Temp = Person(K+Gap)
               do  J = K, 1, -Gap
                  if (Person(J)%Name < Temp%Name) then
                     exit
                  end  if
                  Person(J+Gap) = Person(J)       !! Leapfrog by Gap locations.
               end  do
               Person(J+Gap) = Temp
            end  if
         end  do
         Gap = Gap/2                              !! Halve the gap for the next Ranking Sort.
      end  do
   end  subroutine  SHELL_SORT

!! Figure 1.10  Sorting structures, using the Shell Sort

end  module  GROUP
