module GROUP
   implicit  none
   public :: NAME_SORT

contains

   !! This subroutine sorts an array by sorting continually changing subgroups.
   !! Elements between Person(M) and Person(N) are persons who live in the same suburb, and are
   !! sorted into order, based on NAME.

   subroutine NAME_SORT (Person, M, N)
      !! INCOMING: Person = an array whose elements Person(M) to Person(N) are to be sorted.
      !!           M      = lower subscript of array PERSON;
      !!           N      = upper subscript of array PERSON.
      !! OUTGOING: Person = the sorted array.  Elements Person(M) to Person(N) are in
      !!                    ascending order.

      type (Personage), dimension ( : ), intent (in out) :: Person
      integer, intent (in)         :: M, N

      type (Personage)             :: Temp
      integer                      :: J, K, Gap

      Gap = N/2                                   !! The initial gap is half the number of elements.
      do
         if (Gap < 1) then                        !! We are done after the gap is 1, when all
            exit                                  !! elements are examined in the next loop:
         end  if
         !! Perform a Ranking Sort on selected elements.
         do  K = M, N-Gap
            if (Person(K)%Name > Person(K+Gap)%Name) then
                                                  !! Select elements every Gap positions apart . .
               Temp = Person(K+Gap)
               do  J = K, 1, -Gap
                  if (Person(J)%Name < Temp%Name) then
                     exit
                  end  if
                  Person(J+Gap) = Person(J)       !! Leapfrog by Gap locations.
               end  do
               Person(J+Gap) = Temp
            end  if
         end  do
         Gap = Gap/2                              !! Halve the gap for the next Ranking Sort.
      end  do
   end  subroutine  NAME_SORT

!! Figure 1.11  Sorting structures

end  module  GROUP
