module GROUP
   implicit  none
   public :: DOUBLE_BUBBLE

contains

!! This subroutine implements a Double-Bubble Sort.
!! Method:  The inner loop simultaneously performs a scan from the top to the bottom of the
!! list, and a scan from the bottom to the top.  The scan from the top to the bottom of the
!! list bubbles a large element to the bottom, while the scan commencing at the bottom bubbles
!! a small element from the bottom of the list to the top.  For each swap, variables Top and
!! Bottom are altered to show the location of the most-recent exchange of elements.  When
!! execution of the inner loop terminates, these new values of Top and Bottom determine the
!! extent of the next execution of the inner loop [elements A(lbound(A,1)) to A(Bottom) are
!! then in sort, as are A(Top) to A(ubound(A,1)) ].
!! The outer loop terminates when there is no further change of variable Bottom (which
!! signifies that no values were swapped during a scan), thus behaving like the flag of the
!! Exchange Sort.
subroutine DOUBLE_BUBBLE (A)
   !! INCOMING:   A = an array whose elements are to be sorted;
   !! OUTGOING:   A = the sorted array.  Elements are in ascending order.
   integer, dimension ( : ), intent (in out) :: A

   integer                         :: Temp
   integer                         :: Bottom, Top
   integer                         :: J, K, N
   logical                         :: Sorted

   N = ubound (A, 1)
   Top = N - 1
   Bottom = lbound (A, 1)
   do
      Sorted = .true.
      K = Top

      do J = Bottom, Top
         !! In this loop, the next segment begins at the top & works its way to the bottom.
         if (A(J) > A(J+1)) then                  !! Exchange A(J) with A(J+1).
            Temp = A(J)
            A(J) = A(J+1)
            A(J+1) = Temp
            Top = J                               !! Record the position of the last move.
         end  if
         !! In the same loop, this segment works its way from the bottom to the top.
         if (A(K) > A(K+1)) then                  !! Exchange A(K) with A(K+1).
            Temp = A(K)                           !! Exchange.
            A(K) = A(K+1)
            A(K+1) = Temp
            Bottom = K                            !! Record the position of the last move.
            Sorted = .false.                      !! Raise the flag.
         end  if
         K = K - 1
      end  do
      !! At the end of this loop, elements lbound(A, 1) through A(Bottom) are sorted,
      !! as well as elements A(Top) through ubound(A, 1).
      !! The next iteration only needs to sort A(Bottom) through A(Top).
      if (Sorted) then
         return
      end  if
   end  do

end  subroutine  DOUBLE_BUBBLE

end  module  GROUP
