module GROUP
   implicit  none
   public :: SHELL_SORT

contains

!! This subroutine sorts an array by sorting continually changing subgroups.
subroutine SHELL_SORT (A)
   !! INCOMING:   A = an array whose elements are to be sorted.
   !! OUTGOING:   A = the sorted array.  Elements are in ascending order.
   integer, dimension ( : ), intent (in out) :: A

   integer                         :: Temp
   integer                         :: J, K, N,Gap

   N = ubound(A, 1)                               !! The number of elements in the array A.

   Gap = N/2                                      !! The initial gap is half the number of elements.
   do
      if (Gap < 1) then                           !! We are done after the gap is 1, when all
         exit                                     !! elements are examined in the next loop:
      end  if
      !! Perform a Ranking Sort on selected elements.
      do  K = 1, N-Gap
         if (A(K) > A(K+Gap)) then                !! Select elements every Gap positions apart . .
            Temp = A(K+Gap)
            do  J = K, 1,-Gap
               if (A(J) < Temp) then
                  exit
               end  if
               A(J+Gap) = A(J)                    !! Leapfrog by Gap locations.
            end  do
            A(J+Gap) = Temp
         end  if
      end  do
      Gap = Gap/2                                 !! Halve the gap for the next Ranking Sort.
   end  do

end  subroutine  SHELL_SORT

end  module  GROUP