module  GROUP
   implicit none
   public                          :: COCKTAIL

contains

!! This subroutine uses indirect addressing to sort the data in array A into ascending order.
!! When any two values are out of order, their pointers are exchanged so as to reflect their
!! order.

subroutine COCKTAIL (A, P)
   !! INCOMING:   A = an array whose elements are to be sorted;
   !! OUTGOING:   P = an array of pointers giving the correct order of elements in A.
   !!            The array must be the same size as array A.
   character (len=*), dimension ( : ), intent (in) :: A
   integer, dimension ( : ), intent (out) :: P    !! Pointers to the elements to be sorted.

   integer                         :: N           !! The number of elements in array A.
   integer                         :: Temp        !! For holding a pointer.
   integer                         :: J
   logical                         :: Sorted

   N = ubound(A, 1)                               !! The number of elements to be sorted.

   !! Section to initialize the pointers.
   do J = 1, N
      P(J) = J
   end  do

   !! Section to perform an Indirect Address sort.
   do
      Sorted = .true.                             !! Assume that they are sorted.
      do J = 1, N-1                               !! Forwards.
         if (A(P(J)) > A(P(J+1))) then            !! Swap pointers to reflect the new order of the
                                                  !! elements.
            Temp = P(J)
            P(J) = P(J+1)
            P(J+1) = Temp
            Sorted = .false.                      !! Note that we swapped (raise the flag).
         end  if
      end  do

      if (Sorted) then
         return
      end  if

      do J = N-2, 1, -1                           !! Backwards.
         if (A(P(J)) > A(P(J+1))) then            !! Swap pointers to reflect the new order of the
                                                  !! elements.
            Temp = P(J)
            P(J) = P(J+1)
            P(J+1) = Temp
            Sorted = .false.                      !! Note that we swapped (raise the flag).
         end  if
      end  do

      if (Sorted) then
         return
      end  if

   end  do

end  subroutine  COCKTAIL

end  module  GROUP

program  SORT
   use GROUP
   implicit  none
   integer                         :: N
   character (len=100), dimension ( : ), allocatable  :: A
   integer, dimension ( : ), allocatable        :: P
   integer                         :: J

   print *, "How many elements are there to be sorted?"
   read *, N

   allocate (A(N), P(N) )

   print *, "Please type the elements to be sorted:"
   read *, A

   call COCKTAIL (A, P)

   print *, "The sorted elements are:"
   do J = 1, N
      print *, A(P(J))
   end  do

end  program  SORT

!! Figure 1.7  Main program for the Cocktail-Shaker Sort
