module M2
   implicit  none
   public :: SEARCH

contains

!! This function performs a binary search of a list of elements, held in array A,
!! for the given element, held in Key.
function  SEARCH (A, N, Key) result (Position)
      !! INCOMING: A   = an array of elements;
      !!           N   = the subscript of the last element in the array A.
      !!                 If the elements in A are unchanging, N will be the upper bound of A.
      !!                 If elements are being added to A, N will be less than the upper bound,
      !!                 and elements N+1 to ubound(A,1) will be unused.
      !!           Key = the element for which the search is conducted.
      !! RETURNS:        the position of the element in the array, if it exists;
      !!                 or lbound(A,1)-1, indicating the element does not exist in the array.

      integer, dimension ( : ), intent (in)  :: A
      integer, intent (in)          :: N
      integer, intent (in)          :: Key
      integer                      :: Position

      integer                      :: L, R, Mid

      L = lbound(A, 1) - 1                        !! Set the left pointer.
      R = N + 1                                   !! Set the right pointer.

      do                                          !! While the pointers haven't met . . .
         if (L+1 >= R) then
            exit
         end  if
         Mid = (L+R)/2                            !! Find the mid-point of the reg
         if (Key < A(Mid) ) then                  !! Bring the right pointer to the mid-point.
            R = Mid
         else if (Key > A(Mid) ) then             !! Bring the left pointer to the mid-point.
            L = Mid
         else                                     !! We located the element - Hurrah!
            Position = Mid                        !! Say where it is!
            return                                !! and quit.
         end  if
      end  do

      Position = lbound(A,1)-1                    !! The item was not found in the array.
   end  function  SEARCH

!! Figure 10.2  The binary search algorithm

end  module M2
