module M3
   implicit  none
   public :: SEARCH

contains

!! This subroutine performs a binary search of a list of elements, held in array A,
!! for the given element, held in Key.
subroutine  SEARCH (A, N, Key, Position, Found)
      !! INCOMING: A        = an array of elements;
      !!           N        = the subscript of the last element in the array A.
      !!                      If the elements in A are unchanging, N will be the upper bound of A.
      !!                      If elements are being inserted in A, N will be less than the upper bound,
      !!                      and elements N+1 to UBOUND(A,1) will be unused.
      !!           Key      = the element for which the search is conducted.
      !! OUTGOING: Position = the position of the element in the array, if it exists;
      !!           Found    = .true. if the element exists, .false. otherwise.

      integer, dimension ( : ), intent (in)  :: A
      integer, intent (in)         :: Key
      integer, intent (in)         :: N
      integer, intent (out)        :: Position
      logical, intent (out)        :: Found

      integer                      :: L, R, Mid

      Found = .false.                             !! Preset to say that the element was not found.
      L = lbound(A, 1)                            !! Set the left pointer.
      if (N < L) then                             !! The array is empty.
         return
      end  if
      R = N                                       !! Set the right pointer.

      do
         Mid = (L+R)/2                            !! Find the mid-point of the region.
         if (Key < A(Mid) )then                   !! Bring the right pointer to the mid-point.
            if (L >= R) then                      !! The element was not found in the array A.
               exit
            end  if
            R = Mid
         else if (Key > A(Mid) ) then             !! Bring the left pointer to the mid-point.
            if (L >= R) then                      !! The element was not found in the array A.
               exit
            end  if
            L = Mid+1
         else                                     !! We located the element - Hurrah!
            Position = Mid                        !! Say where it is!
            Found = .true.
            exit
         end  if
      end  do
end  subroutine  SEARCH

!! Figure 10.3  The improved binary search algorithm

end  module M3
