module M6
   implicit  none
   public :: SEARCH

contains

!! This function searches the table T to ascertain whether the element Key exists
!! within it.  If it does exist, the position of the element is returned, otherwise negative is
!! returned.
function  SEARCH (T, Key) result (Position)
      !! INCOMING: T   = an array (table) to be searched.
      !!           Key = the element for which the search is to be conducted.
      !! RETURNS:  the position of the element in the array, if it exists;
      !!           or negative, indicating the element does not exist in the table.

      integer, dimension (0: ), intent (in)  :: T
      integer, intent (in)         :: Key
      integer                      :: Position

      integer                      :: H
      integer, parameter           :: Empty = -1
      integer                      :: N

      N = ubound (T, 1)
      H = modulo(Key, N+1)                        !! Hash the key.

      !! The linear adjustment algorithm.
      do
         if (T(H) == Empty) then
            exit
         end  if
         if (Key == T(H) ) then
            Position = H                          !! The search was successful.
            return
         end  if
         H = H + 1
         if (H > N) then                          !! We reached the end of the table . . .
            H = 0                                 !! . . . so wrap around to the beginning of the table.
         end  if
      end  do

      Position = Empty                            !! The key did not exist in the table.
end  function  SEARCH

!! Figure 10.6  The Hash Search algorithm

end  module  M6
