module M1
   implicit  none
   public :: INTEGRAL

contains

!! This function computes the area under a curve, between two boundaries x=a
!! and x = b, using the Rectangle Method.
function  INTEGRAL (F, A, B, N) result (Area)
      !! INCOMING: F = a procedure that returns the value of the equation y = f(X) at point X;
      !!           A = the left-hand end of the interval to be integrated; and
      !!           B = the right-hand end;
      !!           N = the number of strips in the interval [A, B].
      interface                                   !! for the F/Fortran function defining the function to be integrated.
         function F (X) result (F_Result)
            real, intent (in)      :: X
            real                   :: F_Result
         end  function F
      end  interface
      real, intent (in)            :: A, B
      integer, intent (in)         :: N
      real                         :: Area

      real                         :: Dx, X
      integer                      :: J

      Dx = (B-A)/N                                !! Calculate the strip size.
      Area = 0
      X = A
      do J = 1, N                                 !! A loop to sum the areas of the N rectangles.
         Area = Area + Dx*F(X+J*Dx)               !! Add the area of a rectangle to the partial area.
      end  do

end  function  INTEGRAL

!! Figure 11.1  An F/Fortran function to integrate a given function, using the Rectangle Method.

end  module  M1
