module Funct
   implicit  none
   public                          :: F

contains

   !! A function to evaluate the equation to be solved.
   function F (X, Y) result (Value)
      !! INCOMING: X = the X co-ordinate ) of the point at which the equation is to be evaluated.
      !!           Y = the Y co-ordinate )

      real, intent (in)            :: X, Y
      real                         :: Value

      Value = Y * exp(X)
   end  function  F

end  module  Funct

!! This program solves a differential equation using Euler's method.
program  EULER
   use Funct
   implicit  none
   real                            :: X, Y, deltaX
   integer                         :: K

   print *, "Please type the initial X value:"
   read *, X
   print *, "Please type the corresponding initial Y value:"
   read *, Y

   print *, "Please type the increment delta X:"
   read *, deltaX

   print *, "Iteration   X value         Y value"
   write (unit=*, fmt="(tr7, 2f14.5)" ) X, Y      !! Print the initial values.

   do K = 1, 25
      Y = Y + F(X, Y) * deltaX                    !! Calculate the new value of Y at X = X + deltaX
      X = X + deltaX                              !! Don't forget to increment X.
      write (unit=*, fmt="(i6, tr1, 2f14.5)" ) K, X, Y
                                                  !! Print the iteration count and the (x, y) values for the
                                                  !! new point.
   end  do

end  program  EULER

!! Figure 11.10  Solving a differential equation by Euler's method
