!! This subroutine solves a differential equation using the fourth-order Runge-Kutta method.
subroutine  RUNGE_KUTTA (F, Initial_x, Initial_y, h)
   !! INCOMING: F         = a function to evaluate the equation to be solved;
   !!           Initial_x = the X co-ordinate ) of the initial point.
   !!           Initial_y = the Y co-ordinate )
   !!           h         = the small increment in x at which the function is to be evaluated.

   interface                                      !! for the function to evaluate the equation to be solved.
      function F (X, Y) result (F_Result)
         real, intent (in)         :: X, Y
         real                      :: F_Result
      end  function F
   end  interface
   real, intent (in)               :: Initial_x, Initial_y, h

   real                            :: x, y
   real                            :: k1, k2, k3, k4
   integer                         :: K
   integer, parameter              :: N = 24

   x = Initial_x
   y = Initial_y

   print *, "Iteration   X value         Y value"
   write (unit=*, fmt="(tr7, 2f14.4)" ) Initial_x, Initial_y   !! Print the initial values.

   do K = 1, N
      k1 = h * F(x, y)
      k2 = h * F(x + h/2, y + k1/2)
      k3 = h * F(x + h/2, y + k2/2)
      k4 = h * F(x + h, y + k3)
      y = y + (k1 + 2*k2 + 2*k3 + k4)/6           !! Calculate the new value of Y at X = X + deltaX
      x = x + h                                   !! Don't forget to increment X.

      write (unit=*, fmt="(i6, tr1, 2f14.4)" ) K, x, y
                                                  !! Print the iteration count and the (x, y) values for the
                                                  !! new point.
   end  do

end  subroutine  RUNGE_KUTTA

!! Figure 11.11  Solving a differential equation using the fourth-order Runge-Kutta method
