module M2
   implicit  none
   public :: TRAPEZOID

contains

!! This function computes the area under a curve, between two boundaries x=a
!! and x = b, using the Trapezoidal Method.
function  TRAPEZOID (F, a, b, N) result (Integral)
      !! INCOMING: F = a procedure that returns the value of the equation y = f(X) at point X;
      !!           a = the left-hand end of the interval to be integrated; and
      !!           b = the right-hand end;
      !!           N = the number of strips in the interval [a, b].
      interface                                   !! for the F/Fortran function defining the function to be integrated.
         function F (X) result (F_Result)
            real, intent (in)      :: X
            real                   :: F_Result
         end  function F
      end  interface
      real, intent (in)            :: a, b
      integer, intent (in)         :: N
      real                         :: Integral

      real                         :: Partial_Sum, Dx
      integer                      :: J

      Dx = (b-a)/N
      Partial_Sum = 0
      do J = 1, N-1
         Partial_Sum = Partial_Sum + F(a+J*Dx)
      end  do

      Integral = Dx*(F(a) + F(b))/2 + Partial_Sum*Dx
end  function  TRAPEZOID

!! Figure 11.2  F/Fortran function for numerical integration using the Trapezoidal Method.

end  module  M2
