module M3
   implicit  none
   public :: SIMPSON

contains

!! This function computes the area under a curve, between two boundaries x=a
!! and  x = b, using Simpson's Rule.
function  SIMPSON (F, a, b, N) result (Integral)
      !! INCOMING: F = a procedure that returns the value of the equation y = f(X) at point X;
      !!           a = the left-hand end of the interval to be integrated; and
      !!           b = the right-hand end;
      !!           N = the number of strips in the interval [a, b].
      interface                                   !! for the F/Fortran function defining the function to be integrated.
         function F (X) result (F_Result)
            real, intent (in)      :: X
            real                   :: F_Result
         end  function F
      end  interface
      real, intent (in)            :: a, b
      integer, intent (in)         :: N
      real                         :: Integral

      real                         :: Partial_Sum,      &
                        Sum_odd,    &
                        Sum_even,      &
                        Dx
      integer                      :: J

      Dx = (b-a)/N
      Sum_odd = 0                                 !! Sum the odd-numbered strips.
      do J = 1, N-1, 2
         Sum_odd = Sum_odd + F(a+J*Dx)
      end  do

      Sum_even = 0                                !! Sum the even-numbered strips.
      do J = 2, N-2, 2
         Sum_even = Sum_even + F(a+J*Dx)
      end  do

      Partial_Sum = F(a) + F(b) + 4*Sum_odd + 2*Sum_even
      Integral = Partial_Sum*Dx/3
end  function  SIMPSON

!! Figure 11.3  Numerical integration using Simpson's Rule.

end  module  M3
