module M6
   implicit  none
   public :: DIFFERENTIATE

contains

!! This subroutine computes the differential of a given continuous function F,
!! commencing at the point x=a, finishing at the point x=b, at intervals h.
!! The approximation to the differential is formed in the small interval Dx.
subroutine  DIFFERENTIATE (F, a, b, h, Dx)
      !! INCOMING: F = the function to be differentiated;
      !!           a = the left-hand end of the interval to be differentiated;
      !!           b = the right-hand end of the interval to be differentiated;
      !!           h = the width of each interval; and
      !!           Dx = the small increment/decrement to be used in computing Dy/Dx.
      interface                                   !! for the F/Fortran function defining the function to be differentiated.
         function F (X) result (F_Result)
            real, intent (in)      :: X
            real                   :: F_Result
         end  function F
      end  interface
      real, intent (in)            :: a, b, h, Dx

      real                         :: x, DyDx
      integer                      :: J, N

      N = (b - a)/h                               !! Compute the number of intervals.

      do J = 0, N
         x = a+J*h
         DyDx = (F(x+Dx) - F(x-Dx)) / (2*Dx)
         print *,  "At the point x=", x, ", the differential is ", DyDx
      end  do

end  subroutine  DIFFERENTIATE

!! Figure 11.6  Computing the differential of a given function.

end  module  M6
