module M7
   implicit  none
   public :: NEWTON_RAPHSON

contains

!! This subroutine locates a root of an equation F(x) = 0 using the Newton-Raphson iteration method.
subroutine NEWTON_RAPHSON (F, F_Dash, Root)
   !! INCOMING: F      = an F/Fortran function to evaluate the function F(X) at some point X = X0;
   !!           F_Dash = an F/Fortran function to evaluate F'(X) at some point X = X0;
   !!           Root   = the value of the root, or, in the event of failure, the last iterated value.
   interface                                      !! for the F/Fortran function defining the equation whose root is desired.
      function  F (X) result (Value)
         real, intent (in)         :: X
         real                      :: Value
      end  function  F
      !! And another for the function f'(x):
      function F_Dash (X) result (Value)
         real, intent (in)         :: X
         real                      :: Value
      end  function  F_Dash
   end  interface
   real, intent (out)              :: Root

   real                            :: X0, X1
   integer                         :: J

   print *, "Please type an initial guess for the root:"
   read *, X0

   do J = 1, 30                                   !! Impose a limit of 30 iterations in case of non-convergence.
      X1 = X0 - F(X0)/F_Dash(X0)                  !! Apply one iteration to obtain a new X1 closer to the root.
      if (abs (F(X1)) < 1.0e-10) then             !! Successfully found the root when F(X1) is at or near zero.
         Root = X1
         return
      end  if
      X0 = X1                                     !! Replace the old value in X0 with the new value in X1,
                                                  !! in readiness for the next iteration.
   end  do

   print *, "The algorithm failed to converge.  Final value =", X1
   Root = X1

end  subroutine  NEWTON_RAPHSON

!! Figure 11.7  Locating a root of an equation using the Newton-Raphson Method.

end  module  M7
