module M8
   implicit  none
   public :: LEAST_SQUARES

contains

!! Given: a collection of data points (X(I), Y(I)), this subroutine performs a least-squares
!! approximation, yielding the constants A and B of the equation y = Ax + B.
subroutine  LEAST_SQUARES (X, Y, A, B)
   !! INCOMING: X = an array of X co-ordinates;
   !!           Y = an array of corresponding Y co-ordinates.
   !! OUTGOING: A = the coefficient of the unknown X;
   !!           B = the constant.
   real, dimension ( : ), intent (in)  :: X, Y
   real, intent (out)              :: A, B

   integer                         :: N
   real                            :: Sum_Prods_x_y, Sumx_Sumy, Sum_sqx, Denom

   N = ubound (X, 1)

   Sum_Prods_x_y = sum (X * Y)                    !! Form the sum of the products X(I)*Y(I).

   Sumx_Sumy = sum (X)*sum (Y)                    !! Form the sum of X values multiplied by the
                                                  !! sum of the Y values.

   Sum_sqx = sum (X**2)                           !! Form the sum of the squares of the X values.

   Denom = (N * Sum_sqx - sum (X)**2)

   A = (N * Sum_Prods_x_y - Sumx_Sumy ) / Denom

   B = (Sum_sqx * sum(Y) - Sum_Prods_x_y * sum (X) ) / Denom

end  subroutine  LEAST_SQUARES

!! Figure 11.8  Solving a least-squares approximation.

end  module  M8
