module M9
   implicit  none
   public :: LAGRANGE
   integer, parameter, public :: Double = selected_real_kind (14)
contains

!! This function performs Lagrange interpolation.
function LAGRANGE (X, Y, XW) result (P)
   !! INCOMING: X  = an array of X values; the values must be in increasing order of value.
   !!           Y  = an array of corresponding Y values;
   !!           XW = the X-value for the point at which the interpolated Y-value is to be
   !!                computed.
   !! RETURNS: The interpolated Lagrange value at the point X = XW.

   real (kind=Double), dimension ( : ), intent (in) :: X, Y
   real (kind=Double), intent (in)                  :: XW

   real (kind= Double)             :: P

   real (kind=Double)              :: Partial_Sum, Partial_Product
   integer                         :: I, K, N

   N = ubound(X, 1)

   !! Sum the terms of P(X) = Y(0)*L_0 + Y(1)*L_1 + Y(2)*L_2 + . . .
   Partial_Sum = 0
   do K = 0, N                                    !! For each term in P(X) . . .

      !! Evaluate L_k at the point X = XW.
      Partial_Product = 1
      do I = 0, N                                 !! For each of the "terms" in L_k . . .
         if (I /= K) then
            Partial_Product = Partial_Product * (XW - X(I))/(X(K) - X(I))
         end  if
      end  do
      !! . . . which yields L_k in Partial_Product.

      !! Now add F(X(I))*L_k -- that is, Y(K)*L_k -- to the partial sum for P(X):
      Partial_Sum = Partial_Sum + Y(K)*Partial_Product
   end  do
   !! . . . which yields P(X).

   P = Partial_Sum

end  function  LAGRANGE

!! Figure 11.9  Function for Lagrange Interpolation

end  module  M9
