module M2
   implicit  none
   public :: EXTRACT_SUBMATRIX

contains

   !! This function extracts a sub-matrix from a given matrix argument.
   !! It performs some checks on the arguments before allowing the extraction.
   function  EXTRACT_SUBMATRIX (A, rstart, rfinish, cstart, cfinish) result (B)
      !! INCOMING:   A     = the given matrix;
      !!       rstart   = the first row of the sub-matrix to be extracted;
      !!       rfinish  = the last row of the sub-matrix to be extracted;
      !!       cstart   = the first column of the sub-matrix to be extracted; and
      !!       cfinish  = the last column of the sub-matrix to be extracted.
      !! RETURNS: a sub-matrix extracted from A.

      real, dimension ( : , : ), intent (in)       :: A
      integer, intent (in)                :: rstart, rfinish, cstart, cfinish
      real, dimension (min(rstart, rfinish):max(rstart,rfinish), &
               min(cstart, cfinish):max(cstart,cfinish) )  :: B

      if (rstart  > rfinish) then                 !! First row subscript is greater than the finish
                                                                                !! subscript.
         stop
      end  if
      if (cstart  > cfinish) then                 !! First column subscript is greater than the
                                                                                !! finish subscript.
         stop
      end  if
      if (rstart < lbound(A,1) ) then             !! Row subscript is too small.
         stop
      end  if
      if (cstart < lbound(A,2) ) then             !! Column subscript is too small.
         stop
      end  if
      if (rfinish > ubound(A,1) ) then            !! Row subscript is too large.
         stop
      end  if
      if (cfinish > ubound(A,2) ) then            !! Column subscript is too large.
         stop
      end  if

      B = A(rstart:rfinish, cstart:cfinish)
   end  function  EXTRACT_SUBMATRIX

!! Figure 12.2  Function to extract a sub-matrix.

end  module  M2