!! This special version of DRAW_LINE draws color lines.
!! Assumes that one pixel is stored per byte.
!! This subroutine draws a straight line from the starting point (Xs,Ys) to the
!! finishing point (Xf,Yf).  It also performs scaling of the artwork according to
!! the value of SCALE, and gives it an OFFSET, for centering on the page.
subroutine  DRAW_LINE (Xs, Ys, Xf, Yf)
      !! INCOMING: (Xs, Ys) are the coordinates of the starting point;
      !!           (Xf. Yf) are the coordinates of the finish point.

      integer, intent (in)         :: Xs, Ys,        &
                                      Xf, Yf

      integer, parameter           :: Scaling = 1
      integer, parameter           :: Offset = 0

      real                         :: Row, Col
      integer                      :: X_Increment,   &
                                      Y_Increment
      integer                      :: Byte_Address, Bit_Address, Byte

      if (Ys > Yf) then                           !! "Y_Increment" is 1 Pixel in the Y-direction.
         Y_Increment = -1
      else
         Y_Increment = 1
      end  if
      if (Xs > Xf) then                           !! "X_Increment" is 1 Pixel in the X-direction.
         X_Increment = -1
      else
         X_Increment = 1
      end  if

      if (int(Xf*Scaling) == int(Xs*Scaling) ) then   !! Only the Y-value changes.
         Col = Xs*Scaling + Offset
         Row = Ys*Scaling
!!       do Row = Ys*Scaling, Yf*Scaling, Y_Increment
         do
            if (Y_Increment > 0) then
               if (Row > Yf*Scaling) then
                  exit
               end  if
            else
               if (Row < Yf*Scaling) then
                  exit
               end  if
            end  if

            Pixel_Image(int(Row))(int(Col)+Offset:int(Col)+Offset) = char(Color_Tiff_Green)
            Row = Row + Y_Increment
         end  do
      else if (abs(Xs-Xf) >= abs(Ys-Yf) ) then    !! The slope of the line is <= 45 degrees.
                                                  !! The X and Y-values change, or just X changes.
         Col = Xs*Scaling
!!       do Col = Xs*Scaling, Xf*Scaling, X_Increment
         do
            if (X_Increment > 0) then
               if (Col > Xf*Scaling) then
                  exit
               end  if
            else
               if (Col < Xf*Scaling) then
                  exit
               end  if
            end  if
            Row = (Yf-Ys)*(Col-Xs*Scaling)/(Xf - Xs) + Ys*Scaling

            Pixel_Image(int(Row))(int(Col)+Offset:int(Col)+Offset) = char(Color_Tiff_Green)
            Col = Col + X_Increment
         end  do
      else                                        !! The slope of the line is > 45 degrees.
         Row = Ys*Scaling
!!       do Row = Ys*Scaling, Yf*Scaling, Y_Increment
         do
            if (Y_Increment > 0) then
               if (Row > Yf*Scaling) then
                  exit
               end  if
            else
               if (Row < Yf*Scaling) then
                  exit
               end  if
            end  if
            Col = (Xf-Xs)*(Row-Ys*Scaling)/(Yf - Ys) + Xs*Scaling

            Pixel_Image(int(Row))(int(Col)+Offset:int(Col)+Offset) = char(Color_Tiff_Green)
            Row = Row + Y_Increment
         end  do
      end  if

      return
end  subroutine  DRAW_LINE
