module GROUP
   implicit  none
   public :: SIERPINSKI

contains

!! This procedure draws a Sierpinski curve of order <Order>, one-quarter of the diagram in Figure 2.11.
recursive  subroutine  SIERPINSKI (Direction, Order)
   !! INCOMING: Direction = 0, 1, 2, or 3, representing North, East, South, West respectively,
   !!                       and specifying the direction in which to draw a line of length h pixels.
   !!           Order     = the order (or size) of the Sierpinski curves to be drawn.

   use  COMPASS
   integer, intent (in)            :: Direction, Order
   integer                         :: Dsucc,      &     !! A successor to Direction.
                                      Dpred             !! A predecessor to Direction.

   if (Order <= 0) then
      return
   end  if

   if (Direction == West) then
      Dsucc = North
   else
      Dsucc = Direction + 1
   end  if

   if (Direction == North) then
      Dpred = West
   else
      Dpred = Direction - 1
   end  if

   call SIERPINSKI (Direction, Order-1)
                           call MOVE (Direction, Dsucc, h)
   call SIERPINSKI (Dsucc, Order-1)
                           call MOVE (Direction, Direction, 2*h)
   call SIERPINSKI (Dpred, Order-1)
                           call MOVE (Direction, Dpred, h)
   call SIERPINSKI (Direction, Order-1)

end  subroutine  SIERPINSKI

!! Figure 2.12  Algorithm for generating Sierpinski space-filling curves

end  module  GROUP
