module GROUP
   implicit  none
   public :: MOVE

contains

!! This procedure translates the directions North, East, South, and West to specific
!! increments +h or -h in the X and/or Y directions.  It then calls the procedure DRAW_LINE
!! to draw a straight line from the current point (X0, Y0) to the new point (X0+Dx, Y0+Dy).
subroutine  MOVE (X_Direction, Y_Direction, h)
   !! INCOMING: X_Direction = )-are directions (North, East, South, West) that define the direction
   !!           Y_Direction = )-of the next line to be drawn.  When combined, they produce
   !!                         )-one of the eight movements N, NE, E, SE, S, SW, W, NW.
   !!           h           = the number of units to move in the required direction.

   integer, intent (in)            :: X_Direction, Y_Direction
   integer, intent (in)            :: h

   integer, save                   :: X0 = 10, Y0 = 10  !! The starting point.
   integer                         :: Dx, Dy      !! will hold small increments in the X and Y
                                                  !! directions, respectively, derived from X_direction
                                                  !! & Y_Direction.
                                                  !! X0 & Y0 hold the co-ordinates of the point to
                                                  !! which the previous line was drawn.  When the
                                                  !! next line is drawn, it will start from this point.

   Dx = 0
   Dy = 0
   select case (X_Direction)
      case (North)
         Dy = h
      case (East)
         Dx = h                                   !! Converts East to an increment h in the X direction.
      case (South)
         Dy = -h
      case (West)
         Dx = -h                                  !! Converts West to an increment -h in the X direction.
   end  select

   select case (Y_Direction)
      case (North)
         Dy = h                                   !! Converts North to an increment h in the Y direction.
      case (East)
         Dx = h
      case (South)
         Dy = -h                                  !! Converts South to an increment -h in the Y direction.
      case (West)
         Dx = -h
   end  select

   call DRAW_LINE (X0, Y0, X0+Dx, Y0+Dy)
!! call DRAW_LINE@ (X0, Y0, X0+Dx, Y0+Dy, Color_Green) !! For Salford Graphics

   X0 = X0 + Dx                                   !! Record the new position . . .
   Y0 = Y0 + Dy                                   !! . . . of the co-ordinates,
                                                  !! which become the starting-point of the next line.
end  subroutine  MOVE

!! Figure 2.13  Algorithm for determining the next direction to move.

end  module  GROUP
