module GROUP
   implicit  none
   public :: FILL

contains

!! This procedure fills in the interior of the Sierpinski curves with solid color.
!! The procedure first checks that the pixel is not already colored.  If it is,
!! the procedure terminates.  If the pixel is not colored, it is colored.
!! The procedure then looks in turn at the nearest pixel to the north, west,
!! east, and south.  If the pixel is not colored, the FILL procedure is recursively
!! called with the co-ordinates of that pixel.  Whether or not that pixel was originally
!! colored, the process is repeated with the remaining adjacent pixels.
!! The procedure then terminates.
!! Remove the comment marks at the beginning of Fortran statements for the Salford compiler.
recursive  subroutine  FILL (X, Y, Color)
   !! INCOMING: X     = the X co-ordinate of the pixel to be "filled in";
   !!           Y     = the Y co-ordinate of the pixel to be "filled in";
   !!           Color = the color of the pixel to be "filled in".

   integer, intent (in)            :: X, Y, Color

   integer                         :: Get_Color

   !! Section to ensure that we stay within the area of the graphics image.
   if (X <= lbound(Pixel_Image, 2) .or. X >= ubound(Pixel_Image, 2) ) then
      return
   end  if
   if (Y <= lbound(Pixel_Image, 1) .or. Y >= ubound(Pixel_Image, 1) ) then
      return
   end  if

   Get_Color = ichar (Pixel_Image(Y, X) )         !! Find out the color at the point (X, Y)
!! call GET_PIXEL@ (X, Y, Get_Color)              !! Find out the color at the point (X, Y)
!! Get_Color = iand (Get_Color, 15)               !! Extract the color from the low-order 4 bits.
   if (Get_Color == Color_Tiff_Green .or. Get_Color == Color) then
      return                                      !! Pixel (X, Y) already has a color.
   end  if

!! call SET_PIXEL@ (X, Y, Color)                  !! Color the pixel.
   Pixel_Image(Y, X) = char(Color_Tiff_Red)       !! Red, for TIFF file.

   Get_Color = ichar (Pixel_Image(Y-1, X) )       !! Look at the pixel to the north.
!! call GET_PIXEL@ (X, Y-1, Get_Color)            !! Look at the pixel to the north.
!! Get_Color = iand (Get_Color, 15)               !! Extract the color from the low-order 4 bits.

   if (Get_Color /= Color_Tiff_Green .and. Get_Color /= Color) then
      call FILL (X, Y-1, Color)                   !! Go North.
   end  if

   Get_Color = ichar (Pixel_Image(Y, X-1) )       !! Look at the pixel to the west.
!! call GET_PIXEL@ (X-1, Y, Get_Color)            !! Look at the pixel to the west.
!! Get_Color = iand (Get_Color, 15)               !! Extract the color from the low-order 4 bits.

   if (Get_Color /= Color_Tiff_Green .and. Get_Color /= Color) then
      call FILL (X-1, Y, Color)                   !! Go West, young man.
   end  if

   Get_Color = ichar (Pixel_Image(Y, X+1) )       !! Look at the pixel to the east.
!! call GET_PIXEL@ (X+1, Y, Get_Color)            !! Look at the pixel to the east.
!! Get_Color = iand (Get_Color, 15)               !! Extract the color from the low-order 4 bits.

   if (Get_Color /= Color_Tiff_Green .and. Get_Color /= Color) then
      call FILL (X+1, Y, Color)                   !! Go East.
   end  if

   Get_Color = ichar (Pixel_Image(Y+1, X) )       !! Look at the pixel to the south.
!! call GET_PIXEL@ (X, Y+1, Get_Color)            !! Look at the pixel to the south.
!! Get_Color = iand (Get_Color, 15)               !! Extract the color from the low-order 4 bits.

   if (Get_Color /= Color_Tiff_Green .and. Get_Color /= Color) then
      call FILL (X, Y+1, Color)                   !! Go South.
   end  if

end  subroutine  FILL

!! Figure 2.14  Recursive procedure to fill in the interior of a closed curve

end  module  GROUP
