module GROUP
   implicit  none
   public :: HILBERT

contains

!! This subroutine generates a Hilbert curve.
recursive  subroutine  HILBERT (East, South, West, North, Order)
   !! INCOMING: East, South, West, North specify the direction in which to draw a line of length
   !!           h pixels.
   !!           Order = the order (or size) of the Hilbert curves to be drawn.

   use  COMPASS
   integer, intent (in)            :: East, South, West, North
   integer, intent (in)            :: Order

   integer, parameter              :: h = 15

   if (Order <= 0) then
      return
   end  if

   call HILBERT (South, East, North, West, Order-1)
                                 call MOVE (East, East, h)
   call HILBERT (East, South, West, North, Order-1)
                                 call MOVE (South, South, h)
   call HILBERT (East, South, West, North, Order-1)
                                 call MOVE (West, West, h)
   call HILBERT (North, West, South, East, Order-1)
end  subroutine  HILBERT

!! Figure 2.18  Algorithm for generating Hilbert space-filling curves

end  module  GROUP
