module GROUP
   implicit  none
   public :: MAKE_MAP

contains

!! This subroutine generates the "map" to be used by subroutine HEIGHWAY to generate a Dragon
!! curve.
subroutine  MAKE_MAP (D, Length, Level)
   !! INCOMING: Level  = complexity level of Dragon Curve.
   !! OUTGOING: D      = the bit string for forming the Dragon curve;
   !!           Length = the number of characters in the string D (each character stores a bit).

   character (len=*), intent (out) :: D
   integer, intent (out)           :: Length
   integer, intent (in)            :: Level

   character (len=len(D))          :: E           !! A temporary.
   character (len=1)               :: Bit
   integer                         :: Mid_Bit
   integer                         :: J

   D = "1"
   Length = 1

   do J = 2, Level
      E = D                                       !! Take a copy of the original string.
      Mid_Bit = Length/2 + 1                      !! Calculate the position of the middle bit.
      Bit = E(Mid_Bit: Mid_Bit)

      !! Complement the middle bit of string E
      if (Bit == "1") then
         E(Mid_Bit: Mid_Bit) = "0"
      else
         E(Mid_Bit: Mid_Bit) = "1"
      end  if

      !! Check that D is sufficiently long to store the next level.
      if (len(D) < 2*Length+1) then
         print *, "The input string must be at least", 2*Length+1, &
            " characters long for level", Level
         stop
      end  if

      !! Form the next level.
      D(Length+1: ) = "1" // E                    !! Append "1" and the modified copy of itself, to form
      Length = 2*Length + 1                       !! the next level.
   end  do

end  subroutine  MAKE_MAP

!! Figure 2.20  Generating a map for Dragon Curves

end  module  GROUP
