module GROUP
   implicit  none
   public :: WIRTH

contains

!! This procedure draws a Wirth curve of order <Order>, one-quarter of the diagram in Figure 2.22.
recursive  subroutine  WIRTH (Direction, Order)
   !! INCOMING: Direction = 0, 1, 2, or 3, representing North, East, South, West respectively,
   !!                       and specifying the direction in which to draw a line of length h pixels.
   !!           Order     = the order (or size) of the Wirth curves to be drawn.
   integer, intent (in)            :: Direction, Order

   integer                         :: Dsucc,   &  !! A successor to Direction.
                                      Dpred       !! A predecessor to Direction.

   if (Order <= 0) then
      return
   end  if

   if (Direction == West) then
      Dsucc = North
   else
      Dsucc = Direction + 1
   end  if

   if (Direction == North) then
      Dpred = West
   else
      Dpred = Direction - 1
   end  if

   call WIRTH (Direction, Order-1)
                        call MOVE (Direction, Direction, h)
                        call MOVE (Dsucc, Dsucc, h)
                        call MOVE (Direction, Direction, h)
   call WIRTH (Dsucc, Order-1)
                        call MOVE (Dsucc, Dsucc, h)
                        call MOVE (Direction, Direction, h)
   call WIRTH (Dpred, Order-1)
                        call MOVE (Dpred, Dpred, h)
                        call MOVE (Direction, Direction, h)
                        call MOVE (Dpred, Dpred, h)
   call WIRTH (Direction, Order-1)

end  subroutine  WIRTH

!! Figure 2.23  Subroutine to generate one-quarter of Wirth curves.

end  module  GROUP
