!! This program creates a one-way linked list, and then traverses that list.
!! The program reads in integer values in free format, and stores each value in a node.
!! As each value is read in, a node is created, the value is placed in the node, and the
!! node is attached to the head of the list.  The last of the values read in is zero (which
!! causes the list-creation loop to terminate).  Finally, the list is traversed.

module Node_Definition
   type, public                    :: Node        !! A template for the node & the pointer to it.
      integer                      :: Value       !! For the value.
      type (Node), pointer         :: Ptr         !! For a pointer to the next node.
   end  type  Node
end module Node_Definition

program  CREATE
   use  Node_Definition
   implicit  none
   type (Node), pointer            :: Current     !! An auxiliary pointer.
   type (Node), pointer            :: Head
   integer                         :: Number

   !! The list is created by placing each new item at the head of the list.
   print *, "This program creates a linked list by attaching to the head of the list."
   print *, "Please type the values of each node; the last value is zero:"

   !! SECTION TO CREATE A LINKED LIST.
   nullify (Head)                                 !! Initially the list is empty, so nullify the
                                                  !! pointer to the head of the list.

                                                  !! ATTACH NODES.
   !! A loop to create nodes one-at-a-time, and to attach them to the head of the list.
   do
      read *, Number                              !! Read in a value.
      if (Number == 0) then                       !! Until a sentinel of zero is read in.
         exit
      end  if
      allocate (Current)                          !! Create a new node, to become the head.
      Current%Value = Number                      !! Place the value just read in the new node.
      Current%Ptr => Head                         !! Make the new node point at the head of the
                                                  !! existing list.
      Head => Current                             !! Point at the new node, which has become the
                                                  !! head of the list.
   end  do

   !! SECTION TO TRAVERSE the linked list, printing each element as we go.
   print *, "The elements in the list are:"
   Current => Head                                !! Start at the Head node.
   do
      if (.not. associated(Current)) then         !! Until we reach the tail.
         exit
      end  if
      print *, Current%Value
      Current => Current%Ptr                      !! Point at the next node.
   end  do

end  program  CREATE

!! Figure 3.1  A program to create a linked list and to traverse it.
