module  GROUP
   implicit  none
   public :: ROTATE_LEFT

contains

!! This procedure performs a left rotation of a branch of a Red-Black tree, with node <Pivot>
!! as the pivot.
subroutine  ROTATE_LEFT (Pivot)
      !! INCOMING: Pivot = a pointer to the node about which a branch of the tree is to be rotated.
      use  DEFINE_NODE
      type (Node), pointer         :: Pivot

      type (Node), pointer         :: Y, X

      X => Pivot                                  !! Can't use Pivot directly.  Must use a temporary.

      Y => X%Right                                !! Y is the right child of X.
      X%Right => Y%Left                           !! Change the left subtree of Y into X's right
                                                  !! subtree.
      if (associated(Y%Left) ) then
         Y%Left%Parent => X                       !! The left sub-node of Y points back at X --
                                                  !! that is, X is now the parent of Y's left subtree.
      end  if
      Y%Parent => X%Parent                        !! X's parent now becomes the parent of Y.
      if (.not. associated (X%Parent) ) then      !! We are at the root node.
         Root => Y
      else if (associated (X, X%Parent%Left) ) then   !! X is on the left of its parent.
         X%Parent%Left => Y                       !! The left pointer of X's parent points at Y.
      else                                        !! X is on the right subtree of its parent.
         X%Parent%Right => Y                      !! The right pointer of X's parent points at Y.
      end  if
      Y%Left => X                                 !! Make X a left child of node Y.  (X's left sub-
                                                  !! tree comes with X).
      X%Parent => Y                               !! X's parent is now Y.

end  subroutine  ROTATE_LEFT

!! Figure 3.11  Left rotation of a Red-Black binary tree

end  module  GROUP
