module  GROUP
   implicit  none
   public :: ROTATE_RIGHT

contains

!! This procedure performs a right rotation of a branch of a Red-Black tree, with node <Pivot>
!! as the pivot.  This procedure is the mirror image of ROTATE_LEFT, with the words Left
!! and Right interchanged.
subroutine  ROTATE_RIGHT (Pivot)
      !! INCOMING: Pivot = a pointer to the node about which a branch of the tree is to be rotated.
      use  DEFINE_NODE
      type (Node), pointer         :: Pivot

      type (Node), pointer         :: Y, X

      X => Pivot                                  !! Can't use Pivot directly.  Must use a temporary.

      Y => X%Left                                 !! Y is the left child of X.
      X%Left => Y%Right                           !! Change the right subtree of Y into X's left
                                                  !! subtree.
      if (associated (Y%Right) ) then
         Y%Right%Parent => X                      !! The right sub-node of Y points back at X --
                                                  !! that is, X is now the parent of Y's right subtree
      end  if
      Y%Parent => X%Parent                        !! X's parent now becomes the parent of Y.
      if (.not. associated (X%Parent) ) then      !! We are at the root node.
         Root => Y
      else if (associated (X, X%Parent%Right) ) then  !! X is on the right of its parent.
         X%Parent%Right => Y                      !! The right pointer of X's parent points at Y.
      else                                        !! X is on the left of its parent.
         X%Parent%Left => Y                       !! The left pointer of X's parent points at Y.
      end  if
      Y%Right => X                                !! Put X on the left of Y.
      X%Parent => Y                               !! X's parent is now Y.

end  subroutine  ROTATE_RIGHT

!! Figure 3.12  Right rotation of a Red-Black binary tree

end  module  GROUP
