!! This program creates a one-way linked list by appending to the tail, and then traverses
!! that list.  The program reads in integer values in free format, and stores each value in
!! a node.  As each value is read in, a node is created, the value is placed in the node,
!! and the node is appended to the tail of the list.  The last of the values read in is zero
!! (which causes the list-creation loop to terminate).  Finally, the list is traversed.

module Node_Definition
   type, public                    :: Node        !! A template for the node & the pointer to it.
      integer                      :: Value       !! For the value.
      type (Node), pointer         :: Ptr         !! For a pointer to the next node.
   end  type  Node
end module Node_Definition

program  CREATE
   use Node_Definition
   implicit  none
   type (Node), pointer            :: Current     !! An auxiliary pointer.
   type (Node), pointer            :: Head, &
                                      Tail, &     !! A pointer to the tail of the list.
                                      Old_Tail    !! A temporary to point to the tail of the list.
   integer                         :: Number

   !! The list is created by placing each new item at the tail of the list.
   print *, "This program creates a linked list by appending to the tail."
   print *, "Type the values of each node; the last value is zero:"

   !! SECTION TO CREATE A LINKED LIST.
                                                  !! INITIALIZE AN EMPTY LIST.
   nullify (Head)                                 !! Initially the list is empty, so nullify the
                                                  !! pointer to the head of the list.
   nullify (Tail)                                 !! Likewise, a null pointer for the tail.

                                                  !! APPEND NODES.
   !! A loop to create nodes and to attach them to the tail of the list.
   do
      read *, Number                              !! Read in a value.
      if (Number == 0) then                       !! Until a sentinel of zero is read in.
         exit
      end  if
      Old_Tail => Tail                            !! Make a note of the current tail pointer.
      allocate (Tail)                             !! Create a new node, to become the tail.
      Tail%Value = Number                         !! Place the new value in the new node.
      nullify (Tail%Ptr)                          !! The new tail will have a null pointer.
      if ( .not. associated(Head) ) then          !! The list was empty, so . . .
         Head => Tail                             !! . . . make the Head point at this node too.
      else                                        !! The list was not empty, so . . .
         Old_Tail%Ptr => Tail                     !! . . . make the old tail point at the new tail.
      end  if
   end  do

   !! SECTION TO TRAVERSE the linked list, printing each element as we go.
   print *, "The elements in the list are:"
   Current => Head                                !! Start at the Head node.
   do
      if ( .not. associated(Current)) then        !! Until we reach the tail.
         exit
      end  if
      print *, Current%Value
      Current => Current%Ptr                      !! Point at the next node.
   end  do

end  program  CREATE

!! Figure 3.2  Creating a linked list, by appending to the tail
