   module  Node_Definition
      implicit  none
      type, public                 :: Node
         type (Node), pointer      :: Left
         character (len=1)         :: Symbol
         type (Node), pointer      :: Right
      end  type  Node

   end  module  Node_Definition

   module  GLOBAL
      use  Node_Definition
      private
      character (len=1), public     :: Ch
      type (Node), pointer, public  :: Null_Ptr

   end  module  GLOBAL


   module GROUP
      use  Node_Definition
      implicit none
      private
      public                       :: EXPRESSION, PRIMARY, FACTOR, TERM, ALPHABETIC, ERROR_REPORT
      private                      :: MAKE_SUB_TREE

   contains

      !! This subroutine reads in an algebraic expression, and creates a corresponding binary tree.
      !! An item in the tree can be a letter or an operator (+, -, *, /, ^).
      recursive  subroutine  EXPRESSION (Tptr)
         use  GLOBAL
         type (Node), pointer       :: Tptr

         nullify (Null_Ptr)                       !! Initialize a null pointer.

         read (unit=*, fmt="(a1)", advance="no" ) Ch
         call TERM (Tptr)

         do
            if ((Ch /= "+" ) .and. (Ch /= "-" )) then !! Infix plus and infix minus.
               exit
            end  if
            Tptr => MAKE_SUB_TREE (Tptr, Ch, Null_Ptr )
            read (unit=*, fmt="(a1)", advance="no" ) Ch
            call TERM (Tptr%Right)
         end  do
      end  subroutine  EXPRESSION

      function  MAKE_SUB_TREE (Left_ptr, Ch, Right_ptr) result (MAKE_SUB_TREE_Tptr)
         type (Node), pointer           :: Left_ptr, Right_ptr
         character (len=*), intent (in) :: Ch
         type (Node), pointer           :: MAKE_SUB_TREE_Tptr

         type (Node), pointer           :: Tptr

         allocate (Tptr)
         Tptr%Left         => Left_ptr
         Tptr%Symbol    =  Ch
         Tptr%Right     => Right_ptr
         MAKE_SUB_TREE_Tptr   => Tptr
      end  function  MAKE_SUB_TREE

      recursive  subroutine  PRIMARY (Tptr)
         use  GLOBAL
         type (Node), pointer       :: Tptr

         if (ALPHABETIC (Ch) ) then
            Tptr => MAKE_SUB_TREE (Null_Ptr, Ch, Null_Ptr)
            read (unit=*, fmt="(a1)", advance="no" ) Ch
         else if (Ch == "(" ) then                !! We have a parenthesised (sub) expression . . .
            call EXPRESSION (Tptr)
            if (Ch == ")" ) then                  !! . . . which must finish with a RH parenthesis.
               read (unit=*, fmt="(a1)", advance="no" ) Ch
            else
               call ERROR_REPORT ( "Missing closing parenthesis." )
            end  if
         else
            call ERROR_REPORT ( "A letter or opening parenthesis is expected." )
         end  if
      end  subroutine  PRIMARY

      recursive  subroutine  FACTOR (Tptr)
         use  GLOBAL
         type (Node), pointer       :: Tptr

         call PRIMARY (Tptr)
         do
            if ( Ch /= "^" ) then
               exit
            end  if
            Tptr => MAKE_SUB_TREE (Tptr, Ch, Null_Ptr)
            read (unit=*, fmt="(a1)", advance="no" ) Ch
            call PRIMARY (Tptr%Right)
         end  do
      end  subroutine  FACTOR

      recursive  subroutine  TERM (Tptr)
         use  GLOBAL
         type (Node), pointer       :: Tptr

         call FACTOR (Tptr)
         do
            if ( (Ch /= "*" ) .and. (Ch /= "/" ) ) then
               exit
            end  if
            Tptr => MAKE_SUB_TREE (Tptr, Ch, Null_Ptr)
            read (unit=*, fmt="(a1)", advance="no" ) Ch
            call FACTOR (Tptr%Right)
         end  do
      end  subroutine  TERM

      function  ALPHABETIC (Ch) result (Is_Alphabetic)
         character (len=*), intent (in) :: Ch
         logical                        :: Is_Alphabetic

         character (len=52), parameter :: Letters = "abcdefghijklmnopqrstuvwxyz" // &
                                                    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
         Is_Alphabetic = index (Letters, Ch) > 0
      end  function  ALPHABETIC

      subroutine  ERROR_REPORT (Message)
         character (len=*), intent (in) :: Message

         print *, Message
      end  subroutine  ERROR_REPORT

   end  module  GROUP

!! Figure 3.24  Algorithm for building an expression tree
