module  GROUP
   implicit  none
   public :: SEARCH

contains

!! This function procedure searches a binary search tree for the given element KEY.  It
!! returns the location of the element, or a null pointer if the element could not be found.
function  SEARCH (Key, Root) result (Position)
   !! INCOMING: Key  = the given value to be matched with an element of the tree.
   !!           Root = a pointer to the root of the tree to be searched.
   !! RETURNS:  A pointer to the element (if Key is found in the tree), or a null pointer (if not found)
   use  DEFINE_NODE
   integer, intent (in)            :: Key
   type (Node), pointer            :: Root
   type (Node), pointer            :: Position

   type (Node), pointer            :: Current

   Current => Root

   do
      if (.not. associated(Current ) ) then
         exit
      end if
      if (Current%Item < Key) then                !! Take the left path.
         Current => Current%Left
      else if (Current%Item > Key) then           !! Take the right path.
         Current => Current%Right
      else                                        !! The desired element was found.
         Position => Current
         return
      end  if
   end  do

   nullify (Position)                             !! The element did not exist in the tree.
end  function  SEARCH

!! Figure 3.5  Searching a binary tree

end  module  GROUP
