module  GROUP
   implicit  none
   public :: INSERT_IN_TREE

contains

!! This procedure receives an isolated node NEW, in which all pointers are null, and which
!! contains a value, ready for insertion in the tree.
!! The procedure causes the tree to be searched from the root, along a path determined by
!! the node New's value, until the path terminates in a null pointer.  The node is then
!! inserted in that position, and is colored red.  The pointer to its parent is then
!! initialized.  The pointer V indicates the position of the current node in the tree as the
!! search progresses along a branch.  Pointer W gives the current parent of V.
subroutine  INSERT_IN_TREE (New)
      !! INCOMING: New = A pointer to a node to be inserted into the tree.
      use  DEFINE_NODE
      type (Node), pointer         :: New

      type (Node), pointer         :: V, W

      V => Root                                   !! The initial position of the search.
      nullify (W)                                 !! A pointer to the parent of V.

      !! Find the place in the tree to graft the new node - travel from the root to a node
      !! containing a null pointer.
      do
         if (.not. associated (V) ) then
            exit
         end  if
         W => V
         if (New%Item < V%Item) then              !! Take the left branch.
            V => V%Left
         else                                     !! Take the right branch.
            V => V%Right
         end  if
      end  do
      !! We have found a node W whose left or right pointer field is null.

      New%Parent => W                             !! Make the new node point at its parent.
      if ( .not. associated(W) ) then             !! There's only one node in the tree.
         Root => New
      else if (New%Item < W%Item) then            !! Make the parent point at the new node.
         W%Left => New                            !! The new node is in the left branch.
      else
         W%Right => New                           !! The new node is in the right branch.
      end  if

end  subroutine  INSERT_IN_TREE

!! Figure 3.9  Insertion in a Red-Black binary tree

end  module  GROUP
