!! This subroutine generates Julia set fractals.
subroutine  FRACTALS ( )
   use  GLOBAL
   complex                         :: Z, C
   integer, parameter              :: Screen_Width = 1024, &
                                      Screen_Height =768        !! Measured in pixels.
   integer                         :: Max_Iterations
   integer                         :: Color
   integer                         :: Row, Col
   real                            :: Dx, Dy,           &
                                      Xmin, Ymin,       &
                                      Xmax, Ymax
   real                            :: R, M, M_Squared

   print *, "This program creates a Julia Set fractal, using the equation z = z**2 + c."
   do
      print *, "Please type the fractal constant  c  as a complex number, e.g. (0.25, -0.25) :"

      read *, C
      if (aimag(C) == 0) then
         print *, "The imaginary component must be nonzero.  Please try again."
      else
         exit
      end  if
   end  do

   print *, "Please type the break-out limit (suggest 100):"
   read *, M
   M_Squared = M*M

   print *, "Please type the maximum number of iterations (suggest 30):"
   read *, Max_Iterations

   Xmin = -1.5                                    !! These four lines specify a rectangle in the X-Y
   Xmax = 1.5                                     !! plane.  The fractal will be generated for this
   Ymin = -1.5                                    !! rectangle.  To magnify an area, make the rectangle
   Ymax = 1.5                                     !! small.  The rectangle can be moved around the
                                                  !! X-Y plane by changing any or all of the values.

   Dx = (Xmax - Xmin)/(Screen_Width-1)
   Dy = (Ymax - Ymin)/(Screen_Height-1)

   !! For all the pixels (points) on the screen:
   do Row = 0, Screen_Height-1                    !! . . . in the vertical . . .
      do Col = 0, Screen_Width-1                  !! . . . in the horizontal . . . .
         Z = cmplx (Xmin + Col*Dx, Ymin + Row*Dy) !! Take as the initial value the
                                                  !! co-ordinates of the current point (Row, Col).
         Color = 0                                !! Select Black.

         !! In this loop, the value Z will either escape to infinity, or will remain less than 1.
break_out_loop: do
            Z = Z*Z + C                           !! Transform.
            Color = Color + 1                     !! Select the next color.
            if (Color > Max_Iterations) then      !! The point Z collapsed.
               Color = 0                          !! Select Black.
               exit  break_out_loop
            end  if
            R = real(Z)*real(Z) + aimag(Z)*aimag(Z)
            if (R > M_Squared) then               !! The point Z "escaped" to infinity.
               exit  break_out_loop
            end  if
         end  do  break_out_loop
         if (Color > 7) then
            Color = modulo(Color-1, 7) + 1
         end  if
         Pixel_Image(Row, Col) = char(Color)      !! Color the pixel.
      end  do
   end  do

end  subroutine  FRACTALS

!! Figure 4.11  A program to generate Julia set fractals
