module M15
   implicit  none
   public :: MANDELBROT_FRACTAL

contains

!! This subroutine generates the Mandelbrot Set.
subroutine  MANDELBROT_FRACTAL ( )
   complex                         :: Z, C
   integer, parameter              :: Screen_Width = 640,  &
                                      Screen_Height = 350          !! Measured in pixels.
   integer                         :: Max_Iterations
   integer                         :: Color
   integer                         :: Row, Col
   real                            :: Dx, Dy,           &
                                      Xmin, Ymin,       &
                                      Xmax, Ymax
   real                            :: M, M_squared, R

   print *, "This program creates a Mandelbrot Set fractal, using the equation z = z**2 + c."

   print *, "Now input the dimensions of the Mandelbrot image that you want to view"
   print *, "(for the entire Mandelbrot Set fractal, please type -2, 2, -2, 2):"
   print *, "Please type the left end:"
   read *, Xmin
   print *, "Please type the right-hand end:"
   read *, Xmax
   print *, "Please type the lower end:"
   read *, Ymin
   print *, "Please type the upper end:"
   read *, Ymax

   print *, "Please type the break-out limit (suggest 2):"
   read *, M

   M_squared = M*M

   print *, "Please type the maximum number of iterations:"
   print *, "(suggest 15 for the full Mandelbrot set; 150 for a small part of it):"
   read *, Max_Iterations

   Dx = (Xmax - Xmin)/(Screen_Width-1)
   Dy = (Ymax - Ymin)/(Screen_Height-1)

!! call VGA@                     !! For Salford graphics

   !! For all the pixels (points) on the screen:
   do Row = 0, Screen_Height-1                    !! . . . in the vertical . . .

      do Col = 0, Screen_Width-1                  !! . . . in the horizontal . . . .

         C = cmplx (Xmin + Col*Dx, Ymin + Row*Dy) !! Derive the fractal constant C from the
                                                  !! co-ordinates (Row, Col).
         Z = 0                                    !! Initialize the first Z.

         !! In this loop, the value Z will either escape to infinity, or will remain less than 1.
         Color = 0                                !! Begin with black (Color = 0) . . .
         do
            Z = Z*Z + C                           !! Transform
            R = real(Z)**2 + aimag(Z)**2
            if (R > M_squared) then               !! The point Z "escaped" to infinity.
               exit
            end  if
            Color = Color + 1
            if (Color > Max_Iterations) then      !! The point collapsed to zero . . .
               Color = 0                          !! . . . so color the point black . . .
               exit                               !! . . . and quit.
            end  if
         end  do

         if (Color > 7) then
            Color = modulo(Color-1, 7) + 1
         end  if
         Pixel_Image(Row, Col) = char(Color)      !! Store the color.
!!       call SET_PIXEL@ (Col, Screen_Height-1-Row, Color)  !! For Salford graphics.
      end  do
   end  do

end  subroutine  MANDELBROT_FRACTAL

!! Figure 4.15  Subroutine for generating the Mandelbrot Set

end  module  M15
