!! This program determines the electrical impedance of a pi-pi electrical circuit, which may
!! consist of any number of circuit components in series and parallel.

!! Beginning from the right-hand end of the circuit, the program reads in data for each
!! component in turn, and thus gradually "builds up" the circuit.
!! When the type and value of the next component is read in, it determines the
!! impedance of the circuit (which now consists of the new component and all the components
!! to the right of it).  It does this by using the impedance of the circuit that existed
!! prior to reading in the current component.
!! This continues until the data for all of the components has been read in, by which time
!! the impedance of the entire circuit is known.
program  NETWORK
   implicit  none
   real                            :: R, L, C
   complex                         :: Z, Impedance
   real                            :: Angle, f
   character (len=1)               :: Item, Reply
   logical                         :: Valid, First
   real, parameter                 :: pi = 3.14159

   print *, "This program reads in values of components of a pi network," //  &
         " consisting of resistors, capacitors, and inductors," //      &
         " and determines its impedance."

   print *, "Please type the applied frequency (in Hertz):"
   read *, f
   print *, "The frequency is ", f, " Hz"

   print *, "The component values are required" //             &
         " starting at the right-hand end of the network."
   print *, "Be sure to start at the right-hand end!"

   First = .true.

More: &
   do                                             !! Once around for each component.
      do                                          !! Repeat until the correct component type is given.
         print *, "Please specify the component: L = inductor," //   &
            " C = Capacitor, R = resistor,"
         print *, "     X = complex impedance (in ohms), or Q to quit:"
         read (unit=*, fmt="(a1)" ) Item

         Valid = .true.
         select case (Item)
            case ( "L", "l" )
               print *, "Please type the inductance (in Henries):"
               read *, L
               print *, "The inductance is ", L, " Henries."
               Impedance = cmplx(0.0, 2*pi*f*L)
            case ( "C", "c" )
               print *, "Please type the capacitance (in microFarads):"
               read *, C
               print *, "The capacitance is ", C, " microFarads."
               Impedance = cmplx(0.0, -1.0e6/(2*pi*f*C) )
            case ( "R", "r" )
               print *, "Please type the resistance (in Ohms):"
               read *, R
               print *, "The resistance is ", R, " Ohms."
               Impedance = abs(R)
            case ( "X", "x" )
               print *, "Please type the complex impedance (in Ohms):"
               read *, Impedance
               write (unit=*, fmt="(a, f10.1, spf10.1, a)" ) "The impedance is ", Impedance, "j Ohms."
            case ( "Q", "q" )
               exit More
            case  default
               print *, "Invalid component.  Please try again:"
               Valid = .false.
         end  select
         if (Valid) then
            exit
         end  if
      end  do

      if (First) then                             !! First time only, . . .
         Z = Impedance                            !! . . .the impedance is that of the single component.
         First = .false.
      else
         do                                       !! Repeat until the correct reply is given.
            Valid = .true.
            print *, "Is the item in series or in parallel? (Reply S/P):"
            read (unit=*, fmt="(a1)" ) Reply
            select  case  (Reply)
               case ("S", "s")
                  Z = Z + Impedance
               case ("P", "p")
                  Z = Z * Impedance/(Z + Impedance)
               case default
                  print *, "Incorrect position.  Please try again."
                  Valid = .false.
            end  select
            if (Valid) then
               exit
            end  if
         end  do
      end  if
   end  do  More

   write (unit=*, fmt="(a, f10.1, spf10.1, a)" ) "The impedance of the circuit is ", Z, "j Ohms."

   Angle = atan2 (aimag(Z), real(Z)) * 180 / pi   !! Arctan is in radians, so convert to degrees.
   print *, "The magnitude is ", abs(Z), " Ohms and the phase is ", Angle, " degrees."

end  program  NETWORK

!! Figure 4.20  Calculating the impedance of an electrical network
