!! This program computes the roots of a cubic equation using Cardano's method for complex
!! roots and Viete's method when there are three real roots.
program  CUBIC
   implicit  none
   integer, parameter              :: Double = selected_real_kind (14)
   real                            :: A, B, C, D  !! The coefficients of the cubic.
   real                            :: E, F, G     !! The coefficients of the simplified cubic.
   real                            :: P, Q, R, S
   real                            :: Surd
   real                            :: X1, R2, R3
   complex                         :: X2, X3
   real                            :: Cos3A, Alpha, N
   real (kind=Double), parameter   :: pi = 3.141592653589793_Double

   print *, &
      "This program solves a cubic equation of the form ax**3 + bx**2 + cx + d = 0"
   print *, "Please type the coefficients a, b, c and d in that order:"
   read *, A, B, C, D
   print *, "The coefficients are:  A=", A, " B=", B, " C=", C, " D=", D

   E = B/A                                        !!
   F = C/A                                        !! Convert the coefficients to simpler form.
   G = D/A                                        !!

   S = F/3 - (E/3)**2
   R = (E*F/3 - G)/2 - (E/3)**3
   if (R**2 + S**3 < 0) then                      !! Use Viete's method.
      Q = F - E**2/3
      R = 2*E**3/27 - E*F/3 + G
      !! The equation is now of the form x**3 + qx + r = 0
      N = sqrt(-4*Q/3)
      Cos3A = -4*R* (-3/(4*Q))**1.5
      Alpha = acos(Cos3A)/3
      X1 = N*cos(Alpha) - E/3
      R2 = N*cos(2*pi/3 + Alpha) - E/3
      R3 = N*cos(2*pi/3 - Alpha) - E/3
      print *, "The roots are:  X1=", X1, " R2=", R2, " R3=", R3
      stop
   end  if
   Surd = R + sqrt(R**2 + S**3)
                        P = sign(abs(Surd) ** (1.0/3.0), Surd)
   Surd = R - sqrt(R**2 + S**3)
                        Q = sign(abs(Surd) ** (1.0/3.0), Surd)
   X1 = P + Q - E/3
   X2 = cmplx ( -(P + Q)/2 - E/3, sqrt(0.75)*(P - Q) )
   X3 = conjg(X2)

   print *, "The roots are: X1=", X1
   print *, "X2=", X2
   print *, "X3=", X3
end  program  CUBIC

!! Figure 4.5  A program for producing the roots of a cubic equation
