module  GROUP
	implicit  none

	public :: FFT_COS
	integer, parameter, public :: Double=selected_real_kind (14)

contains

!! This subroutine performs a Fast Fourier cosine transformation of a set of N+1 floating-
!! point data points that are held in the incoming array Y.
subroutine  FFT_COS (Y)
	!! INCOMING: Y = an array of N+1 floating-point data points.  N must be a power of 2.
	!! OUTGOING: Y = the Fourier cosine transform of the data in the incoming array Y.
	!! If this procedure is used to perform the inverse sine transform of data, the outgoing array
	!! Y must be multiplied by 2/N.

		real, dimension ( : ), intent (in out) :: Y

		integer                 :: N
		integer                 :: J
		real                    :: Partial_Sum, Y1, Y2
		real (kind=Double)            :: Theta
		complex (kind = Double)       :: W, WP
		integer                       :: NM1
		real (kind=Double), parameter :: PI = 3.141592653589793_Double

		!! Section to check that N is an integral power of 2.
		N = ubound (Y, 1) -1
		NM1 = N - 1
		if ( (iand(N, NM1) /= 0) .or. (N <= 1) ) then !! N is not a power of 2.
			print *, "The number of data points must be an exact power of 2."
			print *, "The number of data points is ", N
			return
		end  if

		Theta = PI/N
		!! Initialization for the trigonometric recurrence.
		W = 1
		WP = cmplx(-2*sin(Theta/2)**2, sin(Theta), kind=Double)
		Partial_Sum = (Y(1) - Y(N+1))/2
		Y(1) = (Y(1) + Y(N+1))/2
		do J = 1, N/2 - 1
			W = W * WP + W          !! The trigonometric recurrence formula.
			Y1 = (Y(J+1) + Y(N-J+1))/2
			Y2 = Y(J+1) - Y(N-J+1)
			Y(J+1) = Y1 - aimag(W)*Y2
			Y(N-J+1) = Y1 + aimag(W)*Y2
			Partial_Sum = Partial_Sum + real(W)*Y2
		end  do

		call REALFT(Y, N, 1)

		Y(N+1) = Y(2)
		Y(2) = Partial_Sum
		do J = 4, N, 2
			Partial_Sum = Partial_Sum + Y(J)
			Y(J) = Partial_Sum
		end  do

end  subroutine  FFT_COS

!! Figure 5.5  Subroutine to perform a Fast Fourier cosine Transform

end  module  GROUP
