module  M7
   implicit  none

   public :: MMULT

contains

!! This subroutine forms matrix multiplication of the two matrices A and B, and deposits the
!! product in matrix C.
subroutine  MMULT (A, B, C, Error)
   !! INCOMING: A     = a matrix of M rows and N columns, organized as a bit string vector in
   !!                   the N direction;
   !!           B     = a matrix of N rows and K columns, organized as a bit string vector in
   !!                   the N direction;
   !! OUTGOING: C     = a matrix of M rows and K columns, being the
   !!                   matrix product of A with B.
   !!           Error = an error code:  0 = normal case;
   !!                                   2 = incompatible outgoing matrix.

   integer, dimension ( : ), intent (in)      :: A, B
   integer, dimension ( : , : ), intent (out) :: C
   character (len=*), intent (out)            :: Error

   integer                         :: Prod, Partial_Sum
   integer                         :: I, J, K, L, M

   M = ubound (A, 1)                              !! Determine the number of rows in A.
   K = ubound (B, 1)                              !! Determine the number of columns in B.

   J = ubound (C, 1)                              !! Determine the number of rows in C.
   L = ubound (C, 2)                              !! Determine the number of columns in C.
   if ( (J /= M) .or. (L /= K) ) then             !! The outgoing matrix is incompatible with A and B.
      Error = "2"
      return
   end  if

   do I = 1, M
      do J = 1, K
         Prod = iand(A(I), B(J) )                 !! Creates the product as a bit string.
         Partial_Sum = 0
         do                                       !! Count the one-bits in Product.
            Partial_Sum = Partial_Sum + iand(Prod, 1)
            Prod = ishft(Prod, -1)
            if (Prod == 0) then
               exit
            end  if
         end  do
         C(I,J) = Partial_Sum
      end  do
   end  do

   Error = "0"
end  subroutine  MMULT

!! Figure 5.7  Matrix multiplication is much quicker when the elements are zero and unity

end  module  M7
