module M12
   implicit  none
   public :: SEARCH_BAEZA_PERLEBERG

contains

!! The Baeza-Yates-Perleberg approximate string search.
!! All characters of the pattern must be different.
function  SEARCH_BAEZA_PERLEBERG (Text, Pattern, K) result (Number_of_matches)
   !! INCOMING: Text    = the string to be searched;
   !!           Pattern = the pattern we wish to find in TEXT; and
   !!           K       = the number of mismatches allowed.
   !! RETURNS:            the number of matches (with K mismatches).

   character (len=*), intent (in)  :: Text, Pattern
   integer, intent (in)            :: K

   integer                         :: Number_of_matches

   integer, dimension (0:255)      :: Weight
   integer, dimension (0:255)      :: Counter
   integer                         :: M
   integer                         :: I, J, C, Q, R

   M = len (Pattern)
   Counter = M

   !! Preprocessing Section.

   Weight = -1
   do J = len(Pattern), 1, -1
      C = ichar ( Pattern(J:J) )
      Weight (C) = len (Pattern) - J
   end  do

   do J = 0, ubound (Counter, 1)
      if (J < M-1) then
         Counter(J) = ubound(Counter, 1)
      else
         Counter(J) = M
      end  if
   end  do

   !! Begin search.

   Number_of_matches = 0

   do I = 1, len (Text)
      J = ichar (Text(I:I) )
      Q = Weight(J)
      R = iand(I+Q, 255)
      if (Q >= 0) then
         Counter (R) = Counter(R) - 1
      end  if
      if (Counter(iand(I, 255)) <= K) then
         Number_of_matches = Number_of_matches + 1
      end  if
      Counter(iand(I, 255)) = M
   end  do

end  function  SEARCH_BAEZA_PERLEBERG

!! Figure 6.12  Approximate string searching, with k mismatches

end  module  M12
