!! This subroutine extracts the next word from Line, commencing with the CP-th character.
!! The end of the word is indicated by a blank character.
!! When this subroutine terminates, CP is pointing at first blank character at the end of the
!! word (or the end of the line, whichever comes first).

subroutine  EXTRACT_WORD (Line, Word)
   !! INCOMING: Line = the content of a line, as read in.
   !! OUTGOING: Word = the next word extracted from Line.
   use  VARIABLES, only: CP                       !! CP is a pointer to somewhere in Line.
                                                  !! Initially it points to the beginning of Line.
                                                  !! During this procedure, CP points to the initial
                                                  !! letter of a word in Line, then to successive
                                                  !! characters in that word, and finally, to the blan
                                                  !! following that word.  Punctuation characters are
                                                  !! treated as being part of the word.
   character (len=*), intent (in)   :: Line
   character (len=*), intent (out)  :: Word

   integer                         :: J

   !! This section looks for the initial character of the next word.
   do
      if (CP > len (Line) ) then
         exit
      end  if
      if (Line(CP: CP) /= " " ) then
         exit
      end  if
      CP = CP + 1
   end  do

   if (CP > len (Line) ) then                     !! There are no words found during the search, . . .
      Word = " "
      CP = 1
      return                                      !! . . . so return to caller.
   end  if

   !! This section searches for the blank at the end of the word.
   J = index (Line(CP: ), " " )                   !! Look for a blank at the end of the word.
   if (J == 0) then                               !! The word terminates at the end of the line.
      Word = Line(CP: )
      CP = len (Line) + 1                         !! Make CP point past the end of the line.
   else                                           !! The word terminates in a space.
      Word = Line(CP: CP+J-1)
      CP = CP + J                                 !! Adjust CP to point to the blank at the end of the
   end  if                                        !! word.
end  subroutine  EXTRACT_WORD

!! Figure 6.1b  A simple word processor:  A subroutine to extract a word from text.
