!! This subroutine builds lines for the output file, by appending words one-at-a-time to the
!! end of the current output line, until the line becomes full.  Then the line is written
!! out.  One space separates each word, except for the end of a sentence, in which case two
!! blanks are included.
subroutine  BUILD_LINE (Word)
   !! INCOMING:   Word = a word, extracted from Line.
   use  VARIABLES

   character (len=*), intent (in)   :: Word

   character (len=1)               :: Ch
   integer                         :: Spaces
   character (len=2), parameter    :: Blanks = "  "

   !! Section to deal with the first word of a paragraph.
   if (Out_Line(1:Len_Out_Line) == " ") then
      Out_Line(Len_Out_Line+1 : ) = Word
      Len_Out_Line = Len_Out_Line + len_trim (Word)
      return
   end  if

   !! Section to deal with the end of a sentence, indicated by a period (.), ? and !.
   Ch = Out_Line (Len_Out_Line: Len_Out_Line)     !! The last character written out.

   if ( (Ch == "." ) .or. (Ch == "?" ) .or. (Ch == "!" ) ) then !! We are at the end of a sentence.
      Spaces = 2                                  !! Allow two spaces at the end of the sentence.
   else
      Spaces = 1                                  !! Allow one space at the end of the previous word.
   end  if

   !! Section to append the word, and to compute the length of the line.
   if (Len_Out_Line + Spaces + len_trim (Word) > Line_Length) then
                                                  !! The line will overflow.
      write (unit=9, fmt="(a)" ) Out_Line(1:Len_Out_Line)   !! Write out what we have . . .
      Out_Line = Word                             !! and start the word on a fresh line.
      Len_Out_Line = len_trim (Word)
   else
      Out_Line(Len_Out_Line+1 : ) = Blanks(1: Spaces) // Word
                                                  !! Append the word to the line.
      Len_Out_Line = Len_Out_Line + Spaces + len_trim(Word)
   end  if
end  subroutine  BUILD_LINE

!! Figure 6.1c  A simple word processor:  A subroutine to attach a word to an output line.
