module M5
   implicit  none
   public :: SEARCH_KNUTH

contains

!! This function implements the Knuth-Morris-Pratt string search algorithm.
function  SEARCH_KNUTH (String, Pattern) result (Position)
      !! INCOMING: String  = the string to be searched;
      !!           Pattern = the pattern to be used for searching the String.
      !! RETURNS:  The position of Pattern in the String, if successful; or
      !!           zero, indicating an unsuccessful search.
      character (len=*), intent (in)    :: String, Pattern
      integer                           :: Position

      integer, dimension (len(Pattern)) :: Offset
      integer                           :: LP, LS,  &
                                           J, K, T

      !! SETUP SECTION
      !! Associate an integer with each (different) character in the pattern.
      LP = len (Pattern)
      LS = len (String)

      T = 0
      Offset(1) = 0
      do J = 1, LP-1
         do
            if (T <= 0) then
               exit
            end  if
            if (Pattern(J:J) == Pattern(T:T) ) then
               exit
            end  if
            T = Offset(T)
         end  do
         T = T + 1

         if (Pattern(J+1:J+1) /= Pattern(T:T) ) then
            Offset(J+1) = T
         else
            Offset(J+1) = Offset(T)
         end  if
      end  do

      do J = 1, LP
         Offset(J) = J - Offset(J)
      end  do

      !! SEARCH SECTION:  Search the string from left to right.
      K = 1

      do
Inner_Loop: do
            if (K + LP - 1 > LS) then             !! The pattern overlaps the end of the string.
               Position = 0                       !! The search was unsuccesful.
               return
            end  if

            !! COMPARE THE PATTERN WITH THE STRING, FORWARDS.
            do J = 1, LP
               if (Pattern(J:J) /= String(K+J-1:K+J-1) ) then
                                                  !! The pattern does not match this part of the strin
                  K = K + Offset(J)               !! Advance the string pointer according to the value
                                                  !! of Pattern(j).
                  exit  Inner_Loop                !! To compare at the new position of the frame.
               end  if
            end  do

            Position = K                          !! The search was successful.
            return
         end  do  Inner_Loop

      end  do

   end  function  SEARCH_KNUTH

!! Figure 6.5  The Knuth-Morris-Pratt string search

end  module  M5
