   module  GROUP
      implicit  none
      public :: NEXT_TABLE

   contains

      !! Computes the FAIL, NEXT and OFFSET tables for the KMP string search algorithm.
      subroutine  NEXT_TABLE (Pattern)

         character (len=*), intent (in)    :: Pattern

         integer, dimension (len(Pattern)) :: Next  !! The Next table.
         integer, dimension (len(Pattern)) :: f     !! The Fail table.
         integer                           :: j, m, t

         f = 0                                    !! Initialization of the tables.
         Next = 0

         m = len (Pattern)                        !! Position of the end of the pattern.

         f(1) = 0
         Next(1) = 0
         do j = 1, m-1
            t = f(j)
            do
               if (t <= 0) then
                  exit
               end  if
               if (Pattern(j:j) == Pattern(f(j):f(j)) ) then
                  exit
               end  if
               t = Next(t)
            end  do
            f(j+1) = t + 1

            if (Pattern(j+1:j+1) /= Pattern(f(j+1):f(j+1)) ) then
               Next(j+1) = f(j+1)
            else
               Next(j+1) = Next(f(j+1))
            end  if
         end  do

         write (unit=*, fmt="(2a)" ) "Pattern=", Pattern
         write (unit=*, fmt="(a, 20 (i3))" ) "Fail table   ", f
         write (unit=*, fmt="(a, 20 (i3))" ) "Next table   ", Next

         do j = 1, m
            Next(j) = j - Next(j)
         end  do

         write (unit=*, fmt="(a, 20 (i3))" ) "Offset table ", Next

      end  subroutine  NEXT_TABLE

   end  module  GROUP

   program MAKE_NEXT_TABLE
      use  GROUP
      implicit  none
      character (len=100)          :: Pattern

      print *, "Please type the required pattern:"
      read (unit=*, fmt="(a100)" ) Pattern

      call NEXT_TABLE (trim(Pattern) )

   end  program  MAKE_NEXT_TABLE

!! Figure 6.6  Setting up the Fail-Next-Offset tables for the Knuth-Morris-Pratt string search
