module M7
   implicit  none
   public :: SEARCH_BOYER

contains

!! This function implements the "classic" Boyer-Moore string search algorithm.
function  SEARCH_BOYER (String, Pattern) result (Position)
      !! INCOMING: String  = the string to be searched;
      !!           Pattern = the pattern to be used for searching the string.
      !! RETURNS:  The position of Pattern in the String, if successful; or
      !!           zero, indicating an unsuccessful search.

      character (len=*), intent (in) :: String, Pattern
      integer                        :: Position

      integer, dimension (0:255)     :: Offset
      integer                        :: LP, LS,           &
                                        UK, OS, Counter,  &
                                        J, K

      !! SETUP SECTION
      !! Associate an integer with each (different) character in the pattern.
      LP = len (Pattern)
      LS = len (String)
      Offset = 0
      Counter = 1

      do J = LP-1, 1, -1
         UK = ichar (Pattern(J:J))
         if (Offset(UK) == 0) then                !! We have an un-assigned character.
            Offset (UK) = Counter                 !! Associate the character with an integer offset.
         end  if
         Counter = Counter + 1
      end  do
      UK = ichar (Pattern(LP:LP))
      if (Offset(UK) == 0) then                   !! We have an un-assigned character.
         Offset (UK) = Counter                    !! Associate the character with an integer offset.
      end  if
      Counter = Counter + 1

      !! SEARCH SECTION:  Search the string from left to right.
      K = 1
      do
Inner_Loop: do
            if (K + LP - 1 > LS) then             !! The pattern overlaps the end of the string.
               Position = 0                       !! The search was unsuccessful.
               return
            end  if

            !! COMPARE THE PATTERN WITH THE STRING, BACKWARDS.
            do J = LP, 1, -1
               if (Pattern(J:J) /= String(K+J-1:K+J-1) ) then
                                                  !! the pattern does not match this part of the string.
                  OS = Offset (ichar (String(K+J-1:K+J-1)) )
                  if (OS == 0) then               !! Character at K+J-1 in string does not exist in the
                     K = K + J                    !! pattern.  Drag the pattern past the character at
                                                  !! K+J-1.
                  else                            !! Character K+J-1 in string exists in the pattern.
                     K = K + OS                   !! Advance the string pointer according to the
                                                  !! value of String(k), thereby dragging the pattern
                                                  !! so that character at K+J-1 in string matches
                                                  !! Pattern(j).
                  end  if
                  exit  Inner_Loop                !! To compare at the new position of the frame.
               end  if
            end  do

            Position = K                          !! The search was successful.
            return
         end  do  Inner_Loop

      end  do

   end  function  SEARCH_BOYER

!! Figure 6.7  The Boyer-Moore String Search

end  module  M7
