module M9
   implicit  none
   public :: SEARCH_BAEZA

contains

!! The Baeza-Yates & Gonnet string search algorithm.
function  SEARCH_BAEZA (String, Pattern) result (Position)
      !! INCOMING: String  = the string to be searched;
      !!           Pattern = the pattern to be used for searching the string.
      !! RETURNS:  The position of Pattern in the String, if successful; or
      !!           zero, indicating an unsuccessful search.

      character (len=*), intent (in) :: String, Pattern
      integer                      :: Position

      integer, dimension (0:255)   :: T
      integer                      :: State, Lim, Initial
      integer                      :: First
      integer                      :: J
      integer                      :: PPattern, PString, PString_Save !! Pointers to Pattern & String.
      character (len=1)            :: CHP
      integer                      :: UP, UT
      integer                      :: OS
      integer, parameter           :: Max_no_bits = digits(J),   &
                           Largest_Integer = huge(J)

      if (len (Pattern) > len (String) ) then     !! Can't match when the string is longer
         Position = 0                             !! than the pattern.
         return
      end  if
      if ( (len (String) == 0) .or. (len (Pattern) == 0) ) then
         Position = 0                             !! Empty strings can't match anything.
         return
      end  if
      if (len (Pattern) > Max_no_bits) then
            print *, "The pattern cannot exceed ", Max_no_bits, " characters in length"
            stop
      end  if

      !! PREPROCESSING
      T = Largest_Integer
      Lim = 0
      J = 1
      do PPattern = 1, len (Pattern)              !! Sets successive characters to 1, 2, 4, 8, 16, etc.
         UP = ichar (Pattern(PPattern:PPattern))
         T(UP) = iand (T(UP), not(J) )
         Lim = ior (Lim, J)
         J = J * 2                                !! Left shift by one position.
      end  do

      Lim = not(ishft(Lim, -1))                   !! Right shift by one place.
      Lim = iand (Lim, Largest_Integer)           !! Truncate the most significant bit.

      !! SEARCH
      Initial = Largest_Integer
      PPattern = 1
      PString = 1
      First = PPattern
      do
         if (PString > len (String)) then
            exit
         end  if
         CHP = Pattern(First:First)
         PString_Save = PString
         PString = index (String(PString: ), CHP)
         if (PString == 0) then
            Position = 0
            return
         end  if
         PString = PString_Save - 1
         State = Initial
         do                                       !! Drag the pattern past the string.
            UT = ichar (String(PString:PString) )
            OS = ishft(State, 1)                  !! State is shifted left by 1 place, without overflowing.
            OS = iand (OS, Largest_Integer)       !! Truncate the left-most bit.
            State = ior (OS, T(UT) )              !! Raise the duck?
            if (State < Lim) then                 !! We have a match,
               Position = PString - len (Pattern)+1 !! Return the position.
               return
            end  if
            PString = PString + 1                 !! Move the ducks one position to the right.
            if (PString > len (String) ) then     !! All of String examined, without a match.
               Position = 0                       !! Indicate failure to find a match.
               return
            end  if
         end  do
      end  do

      Position = 0                                !! Indicate failure to find a match.
end  function  SEARCH_BAEZA

!! Figure 6.9  The Baeza-Yates & Gonnet string search

end  module  M9
