module  DT_CUSTOMER

   type, public                    :: Customer
      character (len=20)           :: Name
      character (len=20)           :: Street
      character (len=20)           :: Suburb
      character (len=4)            :: Postcode
      character (len=9)            :: Home_Tel_No
      character (len=9)            :: Work_Tel_No
   end  type  Customer

end  module  DT_CUSTOMER

module  GLOBAL

   logical, public                 :: Seen_EOF

end  module  GLOBAL

module GROUP
   implicit none
   public                          :: INFORMATION, SHOW
   private                         :: DETAILS

contains

   !! This subroutine organizes the input of customer details, by supplying a description
   !! of the field to be input, along with the corresponding variable name, to the subroutine
   !! DETAILS.
   subroutine  INFORMATION (Client, Ref_No)
      !! INCOMING: Client = A data structure holding details of a client.
      !!           Ref_No = The reference number for the client.
      !! INCOMING: Client = A data structure holding (possibly) updated details of a client.
      use  DT_CUSTOMER

      type (Customer), intent (in out) :: Client
      integer, intent (out)            :: Ref_No

      print *, "The customer reference number is? :"
      read *, Ref_No
      call DETAILS ( "customer name:", Client%Name)
      call DETAILS ( "house number & street:", Client%Street)
      call DETAILS ( "suburb:", Client%Suburb)
      call DETAILS ( "postcode:", Client%Postcode)
      call DETAILS ( "home telephone number:", Client%Home_Tel_No)
      call DETAILS ( "work telephone number:", Client%Work_Tel_No)
   end  subroutine  INFORMATION

   !! This subroutine requests from the user a specific input.
   subroutine  DETAILS (Category, Item)
      use  GLOBAL
      character (len=*), intent (in)     :: Category
      character (len=*), intent (in out) :: Item

      character (len=20)                 :: Data
      integer                            :: Status

      print *, "Please type the " // Category
      read (unit=*, fmt="(a20)", iostat=Status) Data
      if (Status < 0) then
         Seen_EOF = .true.
         return
      end  if
      Item = adjustl(Data)

   end  subroutine  DETAILS

   !! This procedure prints the content of the file in actual order (that is, in order of
   !! record number.  Empty records are omitted.
   subroutine  SHOW ( )
      use  DT_CUSTOMER
      type (Customer)              :: Client
      type (Customer)              :: Empty
      integer                      :: Status
      integer                      :: Ref_No

      close (unit=9)
      open (unit=9, file="CLIENTS", access = "direct", action="read",   &
            form="unformatted", recl=82, status="old" )

      Empty%Name = repeat (char(0), len(Empty%Name) )
      Ref_No = 1
      print *, "The customer file is:"
      do
         read (unit=9, rec=Ref_No, iostat=Status) Client
         if (Status < 0) then
            exit
         end  if
         if (Client%Name /= Empty%Name) then
            print *, Client
         end  if
         Ref_No = Ref_No + 1
      end  do

   end  subroutine  SHOW

end  module  GROUP

!! This program creates a random access customer file.  When the creation is complete,
!! the program prints the customer file.
program  CREATE
   use  DT_CUSTOMER
   use  GLOBAL
   use  GROUP
   implicit  none
   type (Customer)                 :: Client

   integer                         :: Ref_No

   Seen_EOF = .false.

   open (unit=9, file="CLIENTS", access="direct", action="write",    &
      form="unformatted", recl=82, status="new" )

   do
      print *, "Please type the particulars of a customer:"
      call INFORMATION (Client, Ref_No)
      if (Seen_EOF) then
         exit
      end  if
      write (unit=9, rec=Ref_No) Client
   end  do

   call SHOW ( )

end  program  CREATE

!! Figure 7.4  Creating and updating a random access record-oriented file
