module  GROUP
   implicit  none

   public :: MATINV
   integer, parameter, public :: Double=selected_real_kind (14)

contains

!! This subroutine uses the standard Gauss-Jordan method to invert a matrix.
subroutine  MATINV (A, Error)
      real (kind=Double), dimension ( : , : ), intent (in out) :: A
      character (len=*), intent (out)                          :: Error

      real (kind=Double)           :: Akk
      integer                      :: N
      integer                      :: I, J, K

      Error = "0"                                 !! To anticipate success.
      N = ubound (A, 1)
      if (N /= ubound(A,2) ) then                 !! The matrix is not square.
         Error = "1"
         return
      end  if

      if (N == 1) then                            !! A 1x1 matrix is a simple case.
         A(1,1) = 1/A(1,1)
         return
      end  if

      do K = 1, N
         !! Divide column K by the pivot and negate.
         Akk = A(K,K)
         A( : ,K) = -A( : ,K) / Akk

         do I = 1, N
            !! Effectively subtract a multiple of the pivotal equation K from Equation I.
            if (I /= K) then
               do J = 1, N
                  if (J /= K) then
                     A(I,J) = A(I,K)*A(K,J) + A(I,J)
                  end  if
               end  do
            end  if
         end  do

         !! Divide row K by the pivot.
         A(K, : ) = A(K, : ) / Akk
         A(K,K) = 1/Akk                           !! Replace the pivot by its reciprocal.

      end  do

end  subroutine  MATINV

!! Figure 8.11  Subroutine for inverting a matrix

end  module  GROUP
