module  GROUP
   implicit  none

   public :: READ_BANDED
   integer, parameter, public :: Double=selected_real_kind (14)

contains

!! This subroutine reads in the coefficients and right-hand sides of a banded system of
!! linear equations.
!! Only elements in the band are read in.  Zero elements in the upper & lower triangles
!! are not read in.
subroutine  READ_BANDED (A, R, NDA, NDD)
      !! OUTGOING: A   = the coefficients, in banded form;
      !!           R   = the right-hand sides;
      !!           NDA = the number of nonzero diagonals along the top row;
      !!           NDD = the number of nonzero diagonals down the left-hand column.
      !! NOTE: The calling program must read in the number of equations.

      real (kind=Double), dimension ( : , : ), intent (out) :: A
      real (kind=Double), dimension ( : ), intent (out)     :: R

      integer, intent (out)        :: NDA,     &
                                      NDD

      integer                      :: NDS,     &  !! NDS = number of nonzero diagonals along
                                                  !! the top row;
                                      NDU         !! NDU = number of nonzero diagonals down
                                                  !! the left-hand column.
      integer                      :: C, I, J, N

      !! Perform compatibility checks on the incoming arrays.
      N = ubound(A, 1)                            !! The number of equations.
      if (N /= ubound(A, 2) .or. N /= ubound (R, 1) ) then
         print *, "**ERROR, the dimensions of the arrays for the " //   &
            " banded equation solver are incompatible."
         print *, "The coefficient matrix is of size ", N,              &
            " by ", ubound(A, 2)
         print *, "The vector of right-hand sides is of size ",         &
            ubound (R, 1)
         print *, "The input is abandoned."
         stop
      end  if

      A = 0                                       !! Outside the band must be zero.
      print *, "Please type the number of nonzero diagonals along the top row:"
      read *, NDA
      NDS = NDA
      print *, "Please type the number of nonzero diagonals down the first column:"
      read *, NDD
      NDU = 2 - NDD

      do I = 1, N                                 !! For each row.
         C = max(1, NDU)
         NDS = min(NDS, N)
         print *, "Please type the coefficients for row", I
         do J = C, NDS                            !! For each column.
            read *, A(I,J)
         end  do
         print *, ". . . and the right-hand side:"
         read *, R(I)                             !! The corresponding right-hand side.
         NDS = NDS + 1
         NDU = NDU + 1
      end  do

end  subroutine  READ_BANDED

!! Figure 8.7  Reading in a system of banded equations

end  module  GROUP
