module  GROUP
   implicit  none

   public :: PRINT_BANDED
   integer, parameter, public :: Double=selected_real_kind (14)

contains

   !! This subroutine prints the coefficients and right-hand sides of a banded system of
   !! linear equations.
   !! The diagonals are printed vertically.  Zero elements in the upper triangle and lower
   !! triangle are not printed.
   subroutine  PRINT_BANDED (A, R, NDA, NDD)
      !! INCOMING: A   = the coefficients, in banded form;
      !!           R   = the right-hand sides;
      !!           NDA = the number of nonzero diagonals along the top row;
      !!           NDD = the number of nonzero diagonals down the first column.

      real (kind=Double), dimension ( : , : ), intent (in out) :: A
      real (kind=Double), dimension ( : ), intent (in out)     :: R
      integer, intent (in)                                     :: NDA,  &
                                                                  NDD

      integer                      :: NDS,  &     !! NDS = number of nonzero diagonals along
                                                  !! the top row;
                                      NDU         !! NDU = number of nonzero diagonals down
                                                  !! the left-hand column.
      integer                      :: C, I, J, N

      !! Perform compatibility checks on the incoming arrays.
      N = ubound(A, 1)                            !! The number of equations.
      if (N /= ubound(A, 2) .or. N /= ubound (R, 1) ) then
         print *, "**ERROR, the dimensions of the arrays for the " //   &
            " banded equation printer are incompatible."
         print *, "The coefficient matrix is of size ", N,              &
            " by ", ubound(A, 2)
         print *, "The vector of right-hand sides is of size ",         &
            ubound (R, 1)
         print *, "The output is abandoned."
         stop
      end  if

      NDS = NDA
      NDU = 2 - NDD

      do I = 1, N                                 !! For each row.
         C = max(1, NDU)
         NDS = min (NDS, N)
         print *, "The coefficients for row", I
         do J = C, NDS                            !! For each column.
            write (unit=*, fmt="(f10.6)", advance="no" ) A(I,J)
         end  do
         write (unit=*, fmt="(f10.6)" ) R(I)      !! The corresponding right-hand side.
         NDS = NDS + 1
         NDU = NDU + 1
      end  do

   end  subroutine  PRINT_BANDED

!! Figure 8.8  Subroutine for printing a system of banded equations

end  module  GROUP
